#ifndef __MLIB_BITS_H__
#define __MLIB_BITS_H__

#include <mlib.h>

M_BEGIN_HEADER

/*************************************************************************
 *
 * <Section> bits
 *
 * <Title> Bits
 *
 * <Abstract> Quick ways to handle bits
 *
 * <Description>
 *    The following functions are used to perform common bit-related
 *    operations
 */

 /*************************************************************************
  *
  * @function: m_bits32_highest
  *
  * @description:
  *   this function is used to return the position of the highest bit
  *   that is set to 1 in a 32-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    bit position. -1 if the input value was 0
  */
  MLIB_API(M_Int)   m_bits32_highest( M_UInt32  val );


 /*************************************************************************
  *
  * @function: m_bits16_highest
  *
  * @description:
  *   this function is used to return the position of the highest bit
  *   that is set to 1 in a 16-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    bit position. -1 if the input value was 0
  */
  MLIB_API(M_Int)   m_bits16_highest( M_UInt16  val );



 /*************************************************************************
  *
  * @function: m_bits32_lowest
  *
  * @description:
  *   this function is used to return the position of the lowest bit
  *   that is set to 1 in a 32-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    bit position. -1 if the input value was 0
  */
  MLIB_API(M_Int)   m_bits32_lowest ( M_UInt32  val );


 /*************************************************************************
  *
  * @function: m_bits16_lowest
  *
  * @description:
  *   this function is used to return the position of the lowest bit
  *   that is set to 1 in a 16-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    bit position. -1 if the input value was 0
  */
  MLIB_API(M_Int)   m_bits16_lowest ( M_UInt16  val );



 /*************************************************************************
  *
  * @function: m_bits32_count
  *
  * @description:
  *   this function is used to count the number of bits that are set to 1
  *   in a 32-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    number of set bits (0 to 32)
  */
  MLIB_API(M_UInt)  m_bits32_count  ( M_UInt32  val );


 /*************************************************************************
  *
  * @function: m_bits16_count
  *
  * @description:
  *   this function is used to count the number of bits that are set to 1
  *   in a 16-bit word
  *
  * @input:
  *    val  :: input value
  *
  * @return:
  *    number of set bits (0 to 16)
  */
  MLIB_API(M_UInt)  m_bits16_count  ( M_UInt16  val );



 /*************************************************************************
  *
  * @macro: M_BITS_LSB (x)
  *
  * @description:
  *   returns the least significant bit in x which is set. Note that
  *   this returns a power of 2, not a bit position !!
  */
#define  M_BITS_LSB(x)   ((x) & -(x))


 /*************************************************************************
  *
  * @macro: M_BITS_IS_POWER2 (x)
  *
  * @description:
  *   a macro that returns true iff the input value "x" is a power of 2
  */
#define  M_BITS_IS_POWER2(x)    (((x) & ((x)-1)) == 0)

 /* */

#define  M_BTEST( _lvalue, _flags )    ( (_lvalue) & (_flags) )

#define  M_BSET( _lvalue, _flags )    ( (_lvalue) |= (_flags) )

#define  M_BCLR( _lvalue, _flags )    ( (_lvalue) &= ~(_flags) )

#define  M_FLAG_TEST( _obj, _flags )   M_BTEST( (_obj)->flags, _flags )
#define  M_FLAG_SET( _obj, _flags )    M_BSET ( (_obj)->flags, _flags )
#define  M_FLAG_CLR( _obj, _flags )    M_BCLR ( (_obj)->flags, _flags )

 /* */

M_END_HEADER

#endif /* __MLIB_BITS_H__ */

