#ifndef __MLIB_BYTES_H__
#define __MLIB_BYTES_H__

#include <mlib.h>

M_BEGIN_HEADER

 /***************************************************************************
  *
  * @macro: M_BYTE_P (ptr,offset)
  *
  * @description:
  *    a macro used to access the byte at 'ptr' + 'offset' in memory
  *    'ptr' doesn't need to be a byte pointer.
  *
  *    can be used as a lvalue too
  */
#define  M_BYTE_P(p,i)   (((M_Byte*)(p))[i])


 /***************************************************************************
  *
  * @macro: M_INT8_P (ptr,offset)
  *
  * @description:
  *    a macro used to access the signed byte at 'ptr' + 'offset' in memory
  *    'ptr' doesn't need to be a byte pointer.
  *
  *    can be used as a lvalue too
  */
#define  M_INT8_P(p,i)   (((M_Int8*)(p))[i])


 /***************************************************************************
  *
  * @macro: M_PEEK16_BE (buffer)
  *
  * @description:
  *    peeks the big-endian 16-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK16_BE(p)   ((M_Int16)( (M_INT8_P(p,0) << 8) | \
                                      M_BYTE_P(p,1) ))

 /***************************************************************************
  *
  * @macro: M_PEEK16_UBE (buffer)
  *
  * @description:
  *    peeks the big-endian 16-bit unsigned integer from the memory
  *    location given by "buffer"
  */
#define M_PEEK16_UBE(p)  ((M_UInt16)( (M_BYTE_P(p,0) << 8) | \
                                       M_BYTE_P(p,1) ))

 /***************************************************************************
  *
  * @macro: M_PEEK24_BE (buffer)
  *
  * @description:
  *    peeks the big-endian 24-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK24_BE(p)   ((M_Int32) ( ((M_Int32)M_INT8_P(p,0) << 16) | \
                                      ((M_Int32)M_BYTE_P(p,1) <<  8) | \
                                       (M_Int32)M_BYTE_P(p,2) ))

 /***************************************************************************
  *
  * @macro: M_PEEK24_UBE (buffer)
  *
  * @description:
  *    peeks the big-endian 24-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK24_UBE(p)  ((M_UInt32)( ((M_UInt32)M_BYTE_P(p,0) << 16) | \
                                      ((M_UInt32)M_BYTE_P(p,1) <<  8) | \
                                       (M_UInt32)M_BYTE_P(p,2) ))


 /***************************************************************************
  *
  * @macro: M_PEEK32_BE (buffer)
  *
  * @description:
  *    peeks the big-endian 32-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK32_BE(p)   ((M_Int32) ( ((M_Int32)M_INT8_P(p,0) << 24) | \
                                      ((M_Int32)M_BYTE_P(p,1) << 16) | \
                                      ((M_Int32)M_BYTE_P(p,2) <<  8) | \
                                       (M_Int32)M_BYTE_P(p,3) ))

 /***************************************************************************
  *
  * @macro: M_PEEK32_UBE (buffer)
  *
  * @description:
  *    peeks the big-endian 32-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK32_UBE(p)  ((M_UInt32)( ((M_UInt32)M_BYTE_P(p,0) << 24) | \
                                      ((M_UInt32)M_BYTE_P(p,1) << 16) | \
                                      ((M_UInt32)M_BYTE_P(p,2) <<  8) | \
                                       (M_UInt32)M_BYTE_P(p,3) ))


 /***************************************************************************
  *
  * @macro: M_PEEK32_BE (buffer)
  *
  * @description:
  *    peeks the big-endian 32-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK32_BE(p)   ((M_Int32) ( ((M_Int32)M_INT8_P(p,0) << 24) | \
                                      ((M_Int32)M_BYTE_P(p,1) << 16) | \
                                      ((M_Int32)M_BYTE_P(p,2) <<  8) | \
                                       (M_Int32)M_BYTE_P(p,3) ))

 /***************************************************************************
  *
  * @macro: M_PEEK64_UBE (buffer)
  *
  * @description:
  *    peeks the big-endian 64-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#ifdef M_CONFIG_HAS_INT64

#  define M_PEEK64_UBE(p)  ((M_UInt64)( ((M_UInt64)M_BYTE_P(p,0) << 56) | \
                                        ((M_UInt64)M_BYTE_P(p,1) << 48) | \
                                        ((M_UInt64)M_BYTE_P(p,2) << 40) | \
                                        ((M_UInt64)M_BYTE_P(p,3) << 32) | \
                                        ((M_UInt64)M_BYTE_P(p,4) << 24) | \
                                        ((M_UInt64)M_BYTE_P(p,5) << 16) | \
                                        ((M_UInt64)M_BYTE_P(p,6) <<  8) | \
                                         (M_UInt64)M_BYTE_P(p,7) ))
#endif /* M_CONFIG_HAS_INT64 */

 /***************************************************************************
  *
  * @macro: M_PEEK64_UBE (buffer)
  *
  * @description:
  *    peeks the big-endian 64-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#ifdef M_CONFIG_HAS_INT64

#  define M_PEEK64_BE(p)  ((M_Int64)( ((M_Int64)M_INT8_P(p,0) << 56) | \
                                      ((M_Int64)M_BYTE_P(p,1) << 48) | \
                                      ((M_Int64)M_BYTE_P(p,2) << 40) | \
                                      ((M_Int64)M_BYTE_P(p,3) << 32) | \
                                      ((M_Int64)M_BYTE_P(p,4) << 24) | \
                                      ((M_Int64)M_BYTE_P(p,5) << 16) | \
                                      ((M_Int64)M_BYTE_P(p,6) <<  8) | \
                                       (M_Int64)M_BYTE_P(p,7) ))
#endif /* M_CONFIG_HAS_INT64 */

 /***************************************************************************
  *
  * @macro: M_PEEK16_LE (buffer)
  *
  * @description:
  *    peeks the little-endian 16-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK16_LE(p)   ((M_Int16)( ((M_Int16)M_INT8_P(p,1) << 8) | \
                                      (M_Int16)M_BYTE_P(p,0) ))

 /***************************************************************************
  *
  * @macro: M_PEEK16_ULE (buffer)
  *
  * @description:
  *    peeks the little-endian 16-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK16_ULE(p)  ((M_UInt16)( ((M_UInt16)M_BYTE_P(p,1) << 8) | \
                                       (M_UInt16)M_BYTE_P(p,0) ))

 /***************************************************************************
  *
  * @macro: M_PEEK24_LE (buffer)
  *
  * @description:
  *    peeks the little-endian 24-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK24_LE(p)   ((M_Int32) ( ((M_Int32)M_INT8_P(p,2) << 16) | \
                                      ((M_Int32)M_BYTE_P(p,1) <<  8) | \
                                       (M_Int32)M_BYTE_P(p,0) ))

 /***************************************************************************
  *
  * @macro: M_PEEK24_ULE (buffer)
  *
  * @description:
  *    peeks the little-endian 24-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK24_ULE(p)  ((M_UInt32)( ((M_UInt32)M_BYTE_P(p,2) << 16) | \
                                      ((M_UInt32)M_BYTE_P(p,1) <<  8) | \
                                       (M_UInt32)M_BYTE_P(p,0) ))

 /***************************************************************************
  *
  * @macro: M_PEEK32_LE (buffer)
  *
  * @description:
  *    peeks the little-endian 32-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK32_LE(p)   ((M_Int32) ( ((M_Int32)M_INT8_P(p,3) << 24) | \
                                      ((M_Int32)M_BYTE_P(p,2) << 16) | \
                                      ((M_Int32)M_BYTE_P(p,1) <<  8) | \
                                       (M_Int32)M_BYTE_P(p,0) ))

 /***************************************************************************
  *
  * @macro: M_PEEK32_ULE (buffer)
  *
  * @description:
  *    peeks the little-endian 32-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#define M_PEEK32_ULE(p)  ((M_UInt32)( ((M_UInt32)M_BYTE_P(p,3) << 24) | \
                                      ((M_UInt32)M_BYTE_P(p,2) << 16) | \
                                      ((M_UInt32)M_BYTE_P(p,1) <<  8) | \
                                       (M_UInt32)M_BYTE_P(p,0) ))


 /***************************************************************************
  *
  * @macro: M_PEEK64_LE (buffer)
  *
  * @description:
  *    peeks the little-endian 64-bit signed integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#ifdef M_CONFIG_HAS_INT64

#  define M_PEEK64_LE(p)  ((M_Int64)( ((M_Int64)M_BYTE_P(p,7) << 56) | \
                                      ((M_Int64)M_BYTE_P(p,6) << 48) | \
                                      ((M_Int64)M_BYTE_P(p,5) << 40) | \
                                      ((M_Int64)M_BYTE_P(p,4) << 32) | \
                                      ((M_Int64)M_BYTE_P(p,3) << 24) | \
                                      ((M_Int64)M_BYTE_P(p,2) << 16) | \
                                      ((M_Int64)M_BYTE_P(p,1) <<  8) | \
                                       (M_Int64)M_BYTE_P(p,0) ))
#endif /* M_CONFIG_HAS_INT64 */


 /***************************************************************************
  *
  * @macro: M_PEEK64_ULE (buffer)
  *
  * @description:
  *    peeks the little-endian 64-bit unsigned integer from the memory
  *    location given by "buffer"
  *
  *    "buffer" must be a byte/char pointer
  */
#ifdef M_CONFIG_HAS_INT64

#  define M_PEEK64_ULE(p)  ((M_UInt64)( ((M_UInt64)M_BYTE_P(p,7) << 56) | \
                                        ((M_UInt64)M_BYTE_P(p,6) << 48) | \
                                        ((M_UInt64)M_BYTE_P(p,5) << 40) | \
                                        ((M_UInt64)M_BYTE_P(p,4) << 32) | \
                                        ((M_UInt64)M_BYTE_P(p,3) << 24) | \
                                        ((M_UInt64)M_BYTE_P(p,2) << 16) | \
                                        ((M_UInt64)M_BYTE_P(p,1) <<  8) | \
                                         (M_UInt64)M_BYTE_P(p,0) ))
#endif /* M_CONFIG_HAS_INT64 */


 /***************************************************************************
  *
  * @macro: M_POKE8 (ptr,value)
  *
  * @description:
  *    store a value as an 8-bit byte in memory
  */
#define M_POKE8(p,v)        M_BYTE_P(p,0) = (M_Byte)(v)


 /***************************************************************************
  *
  * @macro: M_POKE16_BE (ptr,value)
  *
  * @description:
  *    store a value as a 16-bit big-endian integer in memory
  */
#define M_POKE16_BE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,1) = (M_Byte)(v);                \
            M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_POKE16_LE (ptr,value)
  *
  * @description:
  *    store a value as a 16-bit little-endian integer in memory
  */
#define M_POKE16_LE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)(v);                \
              M_BYTE_P(p,1) = (M_Byte)((v) >> 16);        \
            M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_POKE24_BE (ptr,value)
  *
  * @description:
  *    store a value as a 24-bit big-endian integer in memory
  */
#define M_POKE24_BE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,1) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,2) = (M_Byte)(v);                \
            M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_POKE24_LE (ptr,value)
  *
  * @description:
  *    store a value as a 24-bit little-endian integer in memory
  */
#define M_POKE24_LE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)(v);                \
              M_BYTE_P(p,1) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,2) = (M_Byte)((v) >> 16);        \
            M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_POKE32_BE (ptr,value)
  *
  * @description:
  *    store a value as a 32-bit big-endian integer in memory
  */
#define M_POKE32_BE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)((v) >> 24);        \
              M_BYTE_P(p,1) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,2) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,3) = (M_Byte)(v);                \
            M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_POKE32_LE (ptr,value)
  *
  * @description:
  *    store a value as a 32-bit big-endian integer in memory
  */
#define M_POKE32_LE(p,v)                                  \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)(v);                \
              M_BYTE_P(p,1) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,2) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,3) = (M_Byte)((v) >> 24);        \
            M_END_STMNT

 /* */
 /***************************************************************************
  *
  * @macro: M_POKE64_BE (ptr,value)
  *
  * @description:
  *    store a value as a 64-bit big-endian integer in memory. only
  *    available on certain platforms
  */
#ifdef M_CONFIG_HAS_INT64
#  define M_POKE64_BE(p,v)                                \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)((v) >> 56);        \
              M_BYTE_P(p,1) = (M_Byte)((v) >> 48);        \
              M_BYTE_P(p,2) = (M_Byte)((v) >> 40);        \
              M_BYTE_P(p,3) = (M_Byte)((v) >> 32);        \
              M_BYTE_P(p,4) = (M_Byte)((v) >> 24);        \
              M_BYTE_P(p,5) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,6) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,7) = (M_Byte)(v);                \
            M_END_STMNT
#endif

 /***************************************************************************
  *
  * @macro: M_POKE64_LE (ptr,value)
  *
  * @description:
  *    store a value as a 64-bit big-endian integer in memory. only
  *    available on certain platforms
  */
#ifdef M_CONFIG_HAS_INT64
#  define M_POKE64_LE(p,v)                                \
            M_BEGIN_STMNT                                 \
              M_BYTE_P(p,0) = (M_Byte)(v);                \
              M_BYTE_P(p,1) = (M_Byte)((v) >>  8);        \
              M_BYTE_P(p,2) = (M_Byte)((v) >> 16);        \
              M_BYTE_P(p,3) = (M_Byte)((v) >> 24);        \
              M_BYTE_P(p,4) = (M_Byte)((v) >> 32);        \
              M_BYTE_P(p,5) = (M_Byte)((v) >> 40);        \
              M_BYTE_P(p,6) = (M_Byte)((v) >> 48);        \
              M_BYTE_P(p,7) = (M_Byte)((v) >> 56);        \
            M_END_STMNT
#endif


/* */

#define  M_POKE16_UBE(p,v)  M_POKE16_BE(p,v)
#define  M_POKE16_ULE(p,v)  M_POKE16_LE(p,v)
#define  M_POKE24_UBE(p,v)  M_POKE24_BE(p,v)
#define  M_POKE24_ULE(p,v)  M_POKE24_LE(p,v)
#define  M_POKE32_UBE(p,v)  M_POKE32_BE(p,v)
#define  M_POKE32_ULE(p,v)  M_POKE32_LE(p,v)

#ifdef M_CONFIG_HAS_INT64
#  define  M_POKE64_UBE(p,v)  M_POKE64_BE(p,v)
#endif

#ifdef M_CONFIG_HAS_INT64
#  define  M_POKE64_ULE(p,v)  M_POKE64_LE(p,v)
#endif

 /***************************************************************************
  *
  * @macro: M_NEXT_INT8 (buffer)
  *
  * @description:
  *    read the next signed byte from "buffer". this macro increments
  *    the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT8( buffer )            \
          ( (M_Int8)*buffer++ )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT8 (buffer)
  *
  * @description:
  *    read the next unsigned byte from "buffer". this macro increments
  *    the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT8( buffer )            \
          ( (M_Byte)*buffer++ )



 /***************************************************************************
  *
  * @macro: M_NEXT_INT16 (buffer)
  *
  * @description:
  *    read the next signed 2-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT16( buffer )     ( buffer += 2, M_PEEK16_BE(buffer-2) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT16 (buffer)
  *
  * @description:
  *    read the next unsigned 2-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT16( buffer )    ( buffer += 2, M_PEEK16_UBE(buffer-2) )


 /***************************************************************************
  *
  * @macro: M_NEXT_INT24 (buffer)
  *
  * @description:
  *    read the next signed 3-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT24( buffer )     ( buffer += 3, M_PEEK24_BE(buffer-3) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT24 (buffer)
  *
  * @description:
  *    read the next unsigned 3-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT24( buffer )   ( buffer += 3, M_PEEK24_UBE(buffer-3) )


 /***************************************************************************
  *
  * @macro: M_NEXT_INT32 (buffer)
  *
  * @description:
  *    read the next signed 4-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT32( buffer )     ( buffer += 4, M_PEEK32_BE(buffer-4) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT32 (buffer)
  *
  * @description:
  *    read the next unsigned 4-byte big-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT32( buffer )     ( buffer += 4, M_PEEK32_UBE(buffer-4) )


 /***************************************************************************
  *
  * @macro: M_NEXT_INT16_LE (buffer)
  *
  * @description:
  *    read the next signed 2-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT16_LE( buffer )   ( buffer += 2, M_PEEK16_LE(buffer-2) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT16_LE (buffer)
  *
  * @description:
  *    read the next unsigned 2-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT16_LE( buffer )  ( buffer += 2, M_PEEK16_ULE(buffer-2) )


 /***************************************************************************
  *
  * @macro: M_NEXT_INT24_LE (buffer)
  *
  * @description:
  *    read the next signed 3-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT24_LE( buffer )   ( buffer += 3, M_PEEK24_LE(buffer-3) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT24_LE (buffer)
  *
  * @description:
  *    read the next unsigned 3-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT24_LE( buffer )   ( buffer += 3, M_PEEK24_ULE(buffer-3) )


 /***************************************************************************
  *
  * @macro: M_NEXT_INT32_LE (buffer)
  *
  * @description:
  *    read the next signed 4-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_INT32_LE( buffer )    ( buffer += 4, M_PEEK32_LE(buffer-4) )


 /***************************************************************************
  *
  * @macro: M_NEXT_UINT32_LE (buffer)
  *
  * @description:
  *    read the next unsigned 4-byte little-endian word starting at "buffer".
  *    this macro increments the pointer.
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_UINT32_LE( buffer )   ( buffer += 4, M_PEEK32_ULE(buffer-4) )


 /***************************************************************************
  *
  * @macro: M_NEXT_BYTE (buffer)
  *
  * @description:
  *    this macro is equivalent to @M_NEXT_UINT8, but is there for
  *    convenience
  *
  * @note:
  *    'buffer' must be a variable whose type is M_Byte*
  */
#define M_NEXT_BYTE(buffer)   ((M_Byte)M_NEXT_UINT8(buffer))


 /***************************************************************************
  *
  * @macro: M_WRITE_BYTE (buffer,value)
  *
  * @description:
  *    a macro used to store a byte at a given memory location, then increment
  *    the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE_BYTE(buffer,b)    M_BEGIN_STMNT                 \
                                    *buffer++ = (M_Byte)(b);    \
                                  M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_WRITE16_BE (buffer,value)
  *
  * @description:
  *    a macro used to store a 16-bit unsigned integer value in big-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE16_BE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE16_BE(buffer,u);        \
                                  buffer   += 2;                \
                                M_END_STMNT

 /***************************************************************************
  *
  * @macro: M_WRITE24_BE (buffer,value)
  *
  * @description:
  *    a macro used to store a 24-bit unsigned integer value in big-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE24_BE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE24_BE(buffer,u);        \
                                  buffer   += 3;                \
                                M_END_STMNT

 /***************************************************************************
  *
  * @macro: M_WRITE32_BE (buffer,value)
  *
  * @description:
  *    a macro used to store a 32-bit unsigned integer value in big-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE32_BE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE32_BE(buffer,u);        \
                                  buffer   += 4;                \
                                M_END_STMNT

 /***************************************************************************
  *
  * @macro: M_WRITE16_LE (buffer,value)
  *
  * @description:
  *    a macro used to store a 16-bit unsigned integer value in little-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE16_LE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE16_LE(buffer,u);        \
                                  buffer   += 2;                \
                                M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_WRITE24_LE (buffer,value)
  *
  * @description:
  *    a macro used to store a 24-bit unsigned integer value in little-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE24_LE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE24_LE(buffer,u);        \
                                  buffer   += 3;                \
                                M_END_STMNT


 /***************************************************************************
  *
  * @macro: M_WRITE32_LE (buffer,value)
  *
  * @description:
  *    a macro used to store a 32-bit unsigned integer value in little-endian
  *    format at a given memory location, then increment the buffer pointer
  *
  * @note:
  *    'buffer' must be a char*, a M_Byte* or a M_Int8*
  */
#define M_WRITE32_LE(buffer,u)  M_BEGIN_STMNT                   \
                                  M_POKE32_LE(buffer,u);        \
                                  buffer   += 4;                \
                                M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP32 (x)
  *
  * @description:
  *   a macro returning a value corresponding to the endian-swap of a
  *   the 32-bit integer 'x'
  */
#define  M_SWAP32(x)   ( (((x) & 0x000000FF) << 24) |  \
                         (((x) & 0x0000FF00) <<  8) |  \
                         (((x) & 0x00FF0000) >>  8) |  \
                         (((x) >> 24) & 0x000000FF) )

 /*************************************************************************
  *
  * @macro: M_SWAP24 (x)
  *
  * @description:
  *   a macro returning a value corresponding to the endian-swap of a
  *   the 24-bit integer 'x'
  */
#define  M_SWAP24(x)   ( (((x) & 0x000000FF) << 16) |  \
                         (((x) & 0x0000FF00)      ) |  \
                         (((x) & 0x00FF0000) >> 16) )

 /*************************************************************************
  *
  * @macro: M_SWAP16 (x)
  *
  * @description:
  *   a macro returning a value corresponding to the endian-swap of
  *   the 16-bit integer 'x'
  */
#define  M_SWAP16(x)   ( (((x) & 0x00FF) << 8) |  \
                         (((x) >> 8) & 0x00FF) )


 /*************************************************************************
  *
  * @macro: M_SWAP_BYTE16 (ptr)
  *
  * @description:
  *   perform an in-place byte-swap of a 16-bit integer value stored at 'ptr'
  *
  *   note that this macro is _not_ atomic, so don't use it on unprotected
  *   shared data.
  *
  *   when you know that 'ptr' is aligned on a 16-bit address, use
  *   @M_SWAP_FAST16 instead
  */
#define  M_SWAP_BYTE16(p)                       \
             M_BEGIN_STMNT                      \
               M_Byte  _temp;                   \
                                                \
               _temp         = M_BYTE_P(p,0);   \
               M_BYTE_P(p,0) = M_BYTE_P(p,1);   \
               M_BYTE_P(p,1) = _temp;           \
             M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP_BYTE24 (ptr)
  *
  * @description:
  *   perform an in-place byte-swap of a 24-bit integer value stored at 'ptr'
  *
  *   note that this macro is _not_ atomic, so don't use it on unprotected
  *   shared data.
  *
  *   when you know that 'ptr' is aligned on a 16-bit address, use
  *   @M_SWAP_FAST16 instead
  */
#define  M_SWAP_BYTE24(p)                       \
             M_BEGIN_STMNT                      \
               M_Byte  _temp;                   \
                                                \
               _temp         = M_BYTE_P(p,0);   \
               M_BYTE_P(p,0) = M_BYTE_P(p,2);   \
               M_BYTE_P(p,2) = _temp;           \
             M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP_BYTE32 (ptr)
  *
  * @description:
  *   perform an in-place byte-swap of a 16-bit integer value stored at 'ptr'
  *
  *   note that hits macro is _not_ atomic, so don't use in on unprotected
  *   shared data.
  *
  *   when you know that 'ptr' is aligned on a 32-bit address, use
  *   @M_SWAP_FAST32 instead.
  */
#define  M_SWAP_BYTE32(p)                        \
             M_BEGIN_STMNT                       \
                M_Byte  _temp1, _temp2;          \
                                                 \
                _temp1        = M_BYTE_P(p,0);   \
                _temp2        = M_BYTE_P(p,1);   \
                M_BYTE_P(p,0) = M_BYTE_P(p,3);   \
                M_BYTE_P(p,1) = M_BYTE_P(p,2);   \
                M_BYTE_P(p,2) = _temp2;          \
                M_BYTE_P(p,3) = _temp1;          \
             M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP_COPY16 (destination,source)
  *
  * @description:
  *   copy a 16-bit value in memory while performing byte-swapping
  */
#define  M_SWAP_COPY16(d,s)                         \
               M_BEGIN_STMNT                        \
                 M_BYTE_P(d,0) = M_BYTE_P(s,1);     \
                 M_BYTE_P(d,1) = M_BYTE_P(s,0);     \
               M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP_COPY24 (destination,source)
  *
  * @description:
  *   copy a 24-bit value in memory while performing byte-swapping
  */
#define  M_SWAP_COPY24(d,s)                         \
               M_BEGIN_STMNT                        \
                 M_BYTE_P(d,0) = M_BYTE_P(s,2);     \
                 M_BYTE_P(d,1) = M_BYTE_P(s,1);     \
                 M_BYTE_P(d,2) = M_BYTE_P(s,0);     \
               M_END_STMNT


 /*************************************************************************
  *
  * @macro: M_SWAP_COPY32 (destination,source)
  *
  * @description:
  *   copy a 32-bit value in memory while performing byte-swapping
  */
#define  M_SWAP_COPY32(d,s)                         \
               M_BEGIN_STMNT                        \
                 M_BYTE_P(d,0) = M_BYTE_P(s,3);     \
                 M_BYTE_P(d,1) = M_BYTE_P(s,2);     \
                 M_BYTE_P(d,2) = M_BYTE_P(s,1);     \
                 M_BYTE_P(d,3) = M_BYTE_P(s,0);     \
               M_END_STMNT


 /* */

#ifdef M_CONFIG_HAS_ENDIANESS

 /*************************************************************************
  *
  * @macro: M_NTOH32 (x)
  *
  * @description:
  *   convert a 32-bit value from network-order (i.e. big-endian) to
  *   host-order (i.e. processor-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_NTOH32(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_NTOH32(x)   M_SWAP32(x)
#endif


 /*************************************************************************
  *
  * @macro: M_NTOH24 (x)
  *
  * @description:
  *   convert a 24-bit value from network-order (i.e. big-endian) to
  *   host-order (i.e. processor-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_NTOH24(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_NTOH24(x)   M_SWAP24(x)
#endif


 /*************************************************************************
  *
  * @macro: M_NTOH16 (x)
  *
  * @description:
  *   convert a 16-bit value from network-order (i.e. big-endian) to
  *   host-order (i.e. processor-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_NTOH16(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_NTOH16(x)   M_SWAP16(x)
#endif


 /*************************************************************************
  *
  * @macro: M_HTON32 (x)
  *
  * @description:
  *   convert a 32-bit value from host-order (i.e. processor-endianess)
  *   to network-order (i.e. big-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_HTON32(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_HTON32(x)   M_SWAP32(x)
#endif


 /*************************************************************************
  *
  * @macro: M_HTON24 (x)
  *
  * @description:
  *   convert a 24-bit value from host-order (i.e. processor-endianess)
  *   to network-order (i.e. big-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_HTON24(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_HTON24(x)   M_SWAP24(x)
#endif


 /*************************************************************************
  *
  * @macro: M_HTON16 (x)
  *
  * @description:
  *   convert a 16-bit value from host-order (i.e. processor-endianess)
  *   to network-order (i.e. big-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if   defined M_CONFIG_BIG_ENDIAN
#  define  M_HTON16(x)   x
#elif defined M_CONFIG_LITTLE_ENDIAN
#  define  M_HTON16(x)   M_SWAP16(x)
#endif


 /*************************************************************************
  *
  * @macro: M_NTOH (x)
  *
  * @description:
  *   convert a int value from network-order (i.e. big-endian) to
  *   host-order (i.e. processor-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if M_SIZEOF_INT == 2
#  define  M_NTOH(x)  M_NTOH16(x)
#elif M_SIZEOF_INT == 4
#  define  M_NTOH(x)  M_NTOH32(x)
#endif


 /*************************************************************************
  *
  * @macro: M_HTON (x)
  *
  * @description:
  *   convert a int value from host-order (i.e. processor-endianess)
  *   to network-order (i.e. big-endianees). This macro is only defined
  *   when @M_CONFIG_HAS_ENDIANESS is defined
  */
#if M_SIZEOF_INT == 2
#  define  M_HTON(x)  M_HTON16(x)
#elif M_SIZEOF_INT == 4
#  define  M_HTON(x)  M_HTON32(x)
#endif


 /* */
#endif  /* M_CONFIG_HAS_ENDIANESS */

M_END_HEADER

#endif /* __MLIB_BYTES_H__ */