/*****************************************************************************
 *
 *  MLib platform-specific configuration file
 *
 *    detection directives for the GNU C compiler
 *
 *  this file is used to compute platform-specific configuration macros,
 *  based on the current compiler setup.
 *
 *  note that we cannot use run-time checks on the host machine to
 *  determine the target endianess.
 *
 *  All configuration macros are _optional_, i.e. they're not required
 *  to compile the MLib. However, certain features or the library will
 *  not be compiled when some "features" are missing
 *
 *  for a list of macros, see the file MLIB_COMPILATION_OPTIONS
 *  (normally <mlib/config/mcompopt.h>)
 *
 *****************************************************************************/

#ifndef __MLIB_COMPILER_GNU_C_H__
#define __MLIB_COMPILER_GNU_C_H__

#ifdef __GNUC__

/* determining processor */

#  if #cpu(386)
#
#    define  M_CPU  M_CPU_X86
#    define  M_CONFIG_HASH_ENDIANESS
#    define  M_CONFIG_LITTLE_ENDIAN
#
#  endif


/* determining endianess if necessary */

#  ifdef __unix__
#
#    define  M_OS   M_OS_UNIX
#
#    ifndef M_CONFIG_HAS_ENDIANESS
#
#      include <endian.h>   // GCC on Unix-like systems should always have this
#
#      if  __BYTE_ORDER == __LITTLE_ENDIAN
#
#        define M_CONFIG_HAS_ENDIANESS
#        define M_CONFIG_LITTLE_ENDIAN
#
#      elif __BYTE_ORDER == __BIG_ENDIAN
#
#        define M_CONFIG_BIG_ENDIAN
#        define M_CONFIG_HAS_ENDIANESS
#
#      endif
#
#    endif
#
#  endif


/* do we have an inline keyword ? */
#  define  M_CONFIG_HAS_INLINE
#  ifdef __STDC__
#    define  M_INLINE  __inline__
#  else
#    define  M_INLINE  inline
#  endif


/* do we have a 64-bits integer type ??             */
/* beware, this is broken on old versions of GCC !! */
/*                                                  */
#  define  M_CONFIG_HAS_INT64
#  define  M_INT64_T      long long
#  define  M_INT64_FMT_0  "ll"

/* do we have the current function's name ? */
#  define  M__FUNCTION__   __FUNCTION__

#endif /* __GNUC__ */

#endif /* __MLIB_COMPILER_GCC_H__ */

