/* project or build specific configuration options */

#ifndef __MLIB_CONFIG_OPTIONS_H__
#define __MLIB_CONFIG_OPTIONS_H__

/******************************************************************
 *
 * @section: configuration
 *
 * @title: Configuration
 *
 * @abstract: Configuration macros
 *
 * @description:
 *    This section contains the list of all configuration macros that
 *    are used at build time to determine which features are turned
 *    "on" or "off" in the MLib.
 *
 *    All configuration macros are placed in the public header file
 *    named "mlib/config/options.h"
 */


/*************************************************************************
 *
 * @macro: M_CONFIG_OPTION_MEMORY_POOLS
 *
 * @description:
 *    define this macro when you want to use fast memory pools to
 *    allocate and release certain common items like list nodes, hash
 *    nodes, etc..
 *
 *    undefine it to only use normal memory allocations. This is useful
 *    when performing debugging with malloc debugging tools (Electric Fence,
 *    DMalloc, Purify, Insure++, CodeGuard, etc..)
 */
#define  M_CONFIG_OPTION_MEMORY_POOLS


/*************************************************************************
 *
 * @macro: M_CONFIG_OPTION_DEBUG_MEMORY
 *
 * @description:
 *    define this macro when you want to use a debugging memory manager
 *    implementation. When the _environment_ variable "MLIB_DEBUG_MEMORY"
 *    is defined, each memory alloc/realloc/free request will be recorded
 *    in order to spot simple but frequent errors like memory leaks and
 *    double deletes.
 *
 *    this also works with custom memory allocators, but requires large
 *    amounts of memory (since "free-ed" blocks are never really released
 *    to the heap during debugging)
 */
#define  M_CONFIG_OPTION_DEBUG_MEMORY


/*************************************************************************
 *
 * @macro: M_CONFIG_OPTION_UNIT_TESTS
 *
 * @description:
 *    define this macro when you want to compile individual unit tests
 *    for all, or nearly all, functions of the MLib. This will build a much
 *    larger library !!
 *
 *    do not #undef it, since it might be selected by the build system
 *    through command line options
 */
#define  M_CONFIG_OPTION_UNIT_TESTS

/*************************************************************************
 *
 * @macro: M_DEBUG_LEVEL_ERROR
 *
 * @description:
 *    define this macro when you want to generate code for the @M_ERROR
 *    macros. They're used to log a serious error within the library
 *    (without halting the application).
 *
 * @note;
 *    the @M_ERROR macros will be compiled anyway if the
 *    @M_DEBUG_LEVEL_LOG macro is defined
 */
#undef   M_DEBUG_LEVEL_ERROR


/*************************************************************************
 *
 * @macro: M_DEBUG_LEVEL_LOG
 *
 * @description:
 *    define this macro when you want to generate code for the @M_LOG
 *    macro and its variants (@M_LOG0, @M_LOG1, etc..). They're used to log
 *    normal operations of the library with various priorities during
 *    execution
 */
#undef   M_DEBUG_LEVEL_LOG


/*************************************************************************
 *
 * @macro: M_DEBUG_COMPILE_ASSERTS
 *
 * @description:
 *    define this macro when you want to generate code for @m_assert
 *    and its variants.
 */
#undef   M_DEBUG_COMPILE_ASSERTS



/*************************************************************************
 *
 * @macro: M_DEBUG_COMPILE_PRECHEKS
 *
 * @description:
 *    define this macro when you want to compile pre-condition checks.
 *    (using the @m_precheck and @m_precheck_val macros), even when
 *    all of @M_DEBUG_COMPILE_ASSERTS, @M_DEBUG_LEVEL_ERROR and
 *    @M_DEBUG_LEVEL_LOG are undefined..
 */
#undef   M_DEBUG_COMPILE_PRECHEKS

/*************************************************************************
 *
 * @macro: M_CONFIG_THREAD_SAFE
 *
 * @description:
 *    this macro should be defined when compiling a thread-safe version
 *    of the MLib.
 */
#define  M_CONFIG_THREAD_SAFE

 /* */

#endif /* __MLIB_CONFIG_OPTIONS_H__ */
