#ifndef __MLIB_CRYPTO_MD5_H__
#define __MLIB_CRYPTO_MD5_H__

M_BEGIN_HEADER

 /****************************************************************
  *
  * @type: M_MD5_State
  *
  * @description:
  *   handle to a @M_MD5_StateRec structure, used to hold the
  *   state of a MD5 hash computation
  */
  typedef struct M_MD5_StateRec_*   M_MD5_State;


 /****************************************************************
  *
  * @struct: M_MD5_StateRec
  *
  * @description:
  *   a structure, used to hold the state of a MD5 hash computation.
  *
  * @fields:
  *    words  :: the current MD5 state
  *    count  :: two 32-bit words to hold the total number of _bits_ sent
  *    buffer :: intermediate 64-byte buffer
  */
  typedef struct M_MD5_StateRec_
  {
    M_UInt32  words[4];
    M_UInt32  count[2];
    M_Byte    buffer[64];

  } M_MD5_StateRec;


 /****************************************************************
  *
  * @function: m_md5_state_init
  *
  * @description:
  *   a function used to initialize a MD5 state for a new computation
  */
  MLIB_API( void )
  m_md5_state_init( M_MD5_State  state );


 /****************************************************************
  *
  * @function: m_md5_state_update
  *
  * @description:
  *   send a given number of bytes to a MD5 hash computation
  *
  * @input:
  *   state :: MD5 state handle
  *   bytes :: input bytes pointer
  *   size  :: number of input bytes
  */
  MLIB_API( void )
  m_md5_state_update( M_MD5_State  state,
                      M_Byte*      bytes,
                      M_Size       size );


 /****************************************************************
  *
  * @function: m_md5_state_final
  *
  * @description:
  *   finalize a MD5 hash computation, and return the final
  *   hash value..
  *
  * @input:
  *   state :: MD5 state handle
  *
  * @output:
  *   md5_hash :: array of 16-bytes
  */
  MLIB_API( void )
  m_md5_state_final( M_MD5_State  state,
                     M_Byte*      md5_hash );

/* */

M_END_HEADER

#endif /* __MLIB_CRYPTO_MD5_H__ */
