/****************************************************************************
 *
 * a generic heap-sort function
 *
 *   heap sort is only marginally slower than quicksort on most cases,
 *   however, it has a guaranteed O(NLogN) worst-case behaviour, instead
 *   of O(N^2) for quick-sort
 *
 *   it is thus the mandatory sorting algorithm for anything that needs
 *   to meet real-time requirements (or simply put it, provide good
 *   responsiveness)
 */

/* disable error message, we don't need to swap for this sort */
#ifndef GEN_CELL_SWAP
#define GEN_CELL_SWAP
#endif

#include <mlib/generic/sort.i>

#ifndef GEN_CELL_MOVE
#define GEN_CELL_MOVE(dst,src)   *(dst) = *(src)
#endif


/***********************************************************************
 *
 * @macro: GEN_SORT_HEAP
 *
 * @description:
 *   when defined, this macro is the name of a function used to implement
 *   a heap-sort algorithmm
 */
#ifdef GEN_SORT_HEAP

#define GEN_SORT_HEAP_SIFT_DEF  M_CONCAT(GEN_SORT_HEAP,_sift)
#define GEN_SORT_HEAP_SIFT(n)   GEN_SORT_HEAP_SIFT_DEF(cells,count,n,&temp GEN_SORT_AUX )


static void
GEN_SORT_HEAP_SIFT_DEF( GEN_SORT_CELLS   cells,
                        GEN_SORT_INDEX   count,
                        GEN_SORT_INDEX   root,
                        GEN_SORT_CELL*   temp
                        GEN_SORT_AUX_    )
{
  GEN_SORT_INDEX  cur, child, right, parent;
  
  /* loop to pull up larger son */
  cur = root;
  for (;;)
  {
    child = cur*2 + 1;  /* left child in heap */
    right = child + 1;  /* right child        */
    if ( child >= count )
      break;

    if ( right < count &&
         GEN_SORT_COMPARE( GEN_SORT_PTR(cells,child),
                           GEN_SORT_PTR(cells,right) ) < 0 )
      child = right;

    GEN_CELL_MOVE( GEN_SORT_PTR(cells,cur), GEN_SORT_PTR(cells,child) );
    cur = child;
  }

  /* loop to check fathers */
  while ( cur != root )
  {
    parent = (GEN_SORT_INDEX)((cur-1)/2);
    if ( GEN_SORT_COMPARE( GEN_SORT_PTR(cells,parent), temp ) >= 0 )
      break;

    GEN_CELL_MOVE( GEN_SORT_PTR(cells,cur),
                   GEN_SORT_PTR(cells,parent) );
    cur = parent;
  }

  /* pop sifted node into place */
  GEN_CELL_MOVE( GEN_SORT_PTR(cells,root), temp );
}



GEN_SORT_DEF( void )
GEN_SORT_HEAP( GEN_SORT_CELLS  cells,
               GEN_SORT_INDEX  count
               GEN_SORT_AUX_   )
{
  GEN_SORT_CELL  temp;
  GEN_SORT_INDEX n;

  if ( count <= 0 )
    return;

  n = count/2 + 1;
  do
  {
    GEN_CELL_MOVE( &temp, GEN_SORT_PTR(cells,n) );
    GEN_SORT_HEAP_SIFT(n);
  }
  while ( --n );

  while ( count > 0 )
  {
    count--;
    GEN_CELL_MOVE( &temp, GEN_SORT_PTR(cells,count) );
    GEN_CELL_MOVE( GEN_SORT_PTR(cells,count), GEN_SORT_PTR(cells,0) );
    GEN_SORT_HEAP_SIFT(1);
  }
}

#undef GEN_SORT_HEAP_SIFT
#undef GEN_SORT_HEAP_SIFT_DEF

#endif /* GEN_SORT_QUICK */

#undef GEN_SORT_HEAP
#undef GEN_CELL_MOVE

#include <mlib/generic/undefs.i>
