/****************************************************************************
 *
 * a generic shell-sort function
 *
 */

#include <mlib/generic/sort.i>

/***********************************************************************
 *
 * @macro: GEN_SORT_SHELL
 *
 * @description:
 *   when defined, this macro is the name of a function used to implement
 *   a shell-sort algorithm
 */
#ifdef GEN_SORT_SHELL

GEN_SORT_DEF( void )
GEN_SORT_SHELL( GEN_SORT_CELLS  cells,
                GEN_SORT_INDEX  count
                GEN_SORT_AUX_   )
{
  GEN_SORT_CELL  *i, *j, *limit = GEN_SORT_PTR(cells,count);
  GEN_SORT_INDEX  gap;

  /* compute initial gap */
  for ( gap = 0; ++gap < count; gap *= 3 )
    ;

  while ( gap /= 3 )
  {
    for ( i = GEN_SORT_PTR(cells,gap); i < limit; GEN_SORT_INCR(i) )
    {
      for ( j = GEN_SORT_PTR(i,-gap); ; j = GEN_SORT_PTR(j,-gap) )
      {
        GEN_SORT_CELL  *k = GEN_SORT_PTR(j,gap);


        if ( GEN_SORT_COMPARE( j, k ) < 0 )
          break;

        GEN_SORT_SWAP( j, k );

        if ( j < GEN_SORT_PTR(cells,gap) )
          break;
      }
    }
  }
}

#endif /* GEN_SORT_SHELL */

#include <mlib/generic/undefs.i>
