/***********************************************************************
 *
 *  you should not include this file directly. Use a specific sort
 *  algorithm template instead..
 *
 *  the following macros definitions are used by all sort algorithm
 *  implementations
 */

/***********************************************************************
 *
 * @macro: GEN_SORT_CELL
 *
 * @description:
 *   this macro holds the type of the sorted cells. You
 *   must always define it before including this file
 */
#ifndef GEN_SORT_CELL
#error  "GEN_SORT_CELL undefined in quick-sort template instantiation"
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_CELLS
 *
 * @description:
 *   this macro holds the type of the input array address. by default,
 *   assume that it is a pointer to GEN_SORT_CELL
 */
#ifndef GEN_SORT_CELLS
#define GEN_SORT_CELLS   GEN_SORT_CELL*
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_PTR (array,index)
 *
 * @description:
 *   this macro is used to compute the address of a given array element
 *   by default, it simply casts the array address to GEN_SORT_CELL*
 */
#ifndef GEN_SORT_PTR
#define GEN_SORT_PTR(_array,_index)  ((GEN_SORT_CELL*)(_array) + (_index))
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_INCR (ptr)
 *
 * @description:
 *   this macro is used to increment a given element pointer during sorting
 */
#ifndef GEN_SORT_INCR
#define GEN_SORT_INCR(_ptr)  (_ptr)++
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_DECR (ptr)
 *
 * @description:
 *   this macro is used to decrement a given element pointer during sorting
 */
#ifndef GEN_SORT_DECR
#define GEN_SORT_DECR(_ptr)  (_ptr)--
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_INDEX
 *
 * @description:
 *   this macro holds the type of the index used to perse the sorted array
 *   by default, this is defined to "unsigned fast" to provide at least
 *   32-bits, but you can also use 'int' when necessary
 */
#ifndef GEN_SORT_INDEX
#define GEN_SORT_INDEX  M_Int
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_COMPARE (cell1,cell2)
 *
 * @description:
 *   this macro is used to compare two cells
 *   before including this file. the result of this macro must be
 *
 */
#ifndef GEN_SORT_COMPARE
#error "GEN_SORT_COMPARE undefined in sort template instantiation"
#endif


/***********************************************************************
 *
 * @macro: GEN_SORT_SWAP (cell1,cell2,temp)
 *
 * @description:
 *   this macro is used to swap two cells in-place during a sort
 */
#ifndef GEN_SORT_SWAP
#define GEN_SORT_SWAP(c1,c2)        \
            M_BEGIN_STMNT           \
              GEN_SORT_CELL  temp;  \
              temp  = *(c1);        \
              *(c1) = *(c2);        \
              *(c2) = temp;         \
            M_END_STMNT
#endif

/***********************************************************************
 *
 * @macro: GEN_SORT_AUX_
 *
 * @description:
 *   this macro can be re-defined in order to allow additional arguments
 *   to the sort function templates.. don't forget that if you
 *   re-define it, it's first character must be a _comma_ (,) !!
 *
 *   see also @GEN_SORT_AUX
 */
#ifndef GEN_SORT_AUX_
#define GEN_SORT_AUX_   /* nothing */
#endif


/***********************************************************************
 *
 * @macro: GEN_SORT_AUX
 *
 * @description:
 *   this macro must be redefined when @GEN_SORT_AUX_ is defined.
 *   it is used to specify function parameters used to _call_ a helper
 *   function for the sort function template
 *
 *   for example, if you want to pass a "data" argument to your comparison
 *   function, do something like:
 *
 *   {
 *     ....
 *     #define GEN_CELL_COMPARE(cell1,cell2)  my_compare_func( cell1, cell2, mydata )
 *     #define GEN_SORT_AUX   , mydata
 *     #define GEN_SORT_AUX_  , M_Pointer  mydata
 *
 *     #include <mlib/generic/heapsort.i>
 *   }
 *
 *   yes, it must begin with a comma (,), when not void !!
 */
#ifndef GEN_SORT_AUX
#define GEN_SORT_AUX  /* nothing */
#endif




/* */
#ifndef GEN_SORT_DEF
#define GEN_SORT_DEF(x)   static x
#endif
