/***************************************************************************
 *
 * generic vector template
 *
 */

#ifndef GEN_NODE
#define GEN_NODE  /* nothing */
#endif

#include <mlib/generic/contain.i>

#ifndef GEN_ITEM
#error "GEN_ITEM not defined in vector template instantiation"
#endif

#ifndef GEN_ITEM_0
#define GEN_ITEM_0  0
#endif

#ifndef GEN_VECTOR_COUNT
#define GEN_VECTOR_COUNT(_table)   (_table)->num_items
#endif

#ifndef GEN_VECTOR_SIZE
#define GEN_VECTOR_SIZE(_table)    (_table)->max_items
#endif

#ifndef GEN_VECTOR_ITEMS
#define GEN_VECTOR_ITEMS(_table)   (_table)->items;
#endif

#ifndef GEN_VECTOR_INCREMENT
#define GEN_VECTOR_INCREMENT(_table)    8
#endif

#ifndef GEN_ITEM_SET
#define GEN_ITEM_SET(_item,_vector)   M_DUMMY_STMNT
#endif

#ifndef GEN_ITEM_UNSET
#define GEN_ITEM_UNSET(_item,_vector)  M_DUMMY_STMNT
#endif


#ifndef GEN_ITEM_COMPARE
#if defined(GEN_VECTOR_QSORT_DEF) || defined(GEN_VECTOR_HEAPSORT_DEF)
#error "GEN_ITEM_COMPARE undefined in vector sort template instantiation"
#endif
#endif


#ifndef GEN_INDEX
#define GEN_INDEX   M_Int
#endif

#ifndef GEN_UINDEX
#define GEN_UINDEX  M_UInt
#endif



#ifndef GEN_VECTOR_ENSURE
#define GEN_VECTOR_ENSURE  GEN_TABLE_FUNC(_ensure)
#endif

#ifdef GEN_VECTOR_ENSURE_DEF

GEN_DEF( void )
GEN_VECTOR_ENSURE( GEN_TABLE   table,
                   GEN_UINDEX  count )
{
  GEN_UINDEX  old_max = GEN_VECTOR_SIZE(table);

  if ( count > old_max )
  {
    GEN_UINDEX  increment = GEN_VECTOR_INCREMENT(table);
    GEN_UINDEX  delta     = increment > 0 ? ( count % increment ) : 0;
    GEN_UINDEX  new_max;


    increment = GEN_VECTOR_INCREMENT(table);
    delta     = increment > 0 ? (count % increment) : 0;
    new_max   = ( delta > 0 ) ? count + increment - delta : count;

    GEN_TABLE_REALLOC( GEN_VECTOR_ITEMS(table),
                       old_max * sizeof(GEN_ITEM),
                       new_max * sizeof(GEN_ITEM),
                       table );

    GEN_VECTOR_SIZE(table) = new_max;
  }
}

#undef GEN_VECTOR_ENSURE_DEF
#endif



#ifdef GEN_VECTOR_CLEAR_DEF

#ifndef GEN_VECTOR_CLEAR
#define GEN_VECTOR_CLEAR  GEN_TABLE_FUNC(_clear)
#endif

GEN_DEF(void)
GEN_VECTOR_CLEAR( GEN_TABLE  table )
{
  if ( table )
  {
    GEN_ITEM    item, *items;
    GEN_UINDEX  count;

    GEN_TABLE_ASSERT(table);

    count = GEN_VECTOR_COUNT(table);
    items = GEN_VECTOR_ITEMS(table);
    for ( ; count > 0; count--, items++ )
    {
      item     = items[0];
      items[0] = GEN_ITEM_0;
      GEN_ITEM_UNSET(item,table);
    }

    GEN_VECTOR_COUNT(table) = 0;
  }
}

#undef GEN_VECTOR_CLEAR_DEF
#undef GEN_VECTOR_CLEAR
#endif




#ifdef GEN_VECTOR_INSERT_DEF

#ifndef GEN_VECTOR_INSERT
#define GEN_VECTOR_INSERT  GEN_TABLE_FUNC(_insert)
#endif

GEN_DEF( void )
GEN_VECTOR_INSERT( GEN_TABLE  table,
                   GEN_INDEX  index,
                   GEN_ITEM   item )
{
  GEN_UINDEX  count, len;
  GEN_ITEM*   items;

  GEN_TABLE_ASSERT(table);

  count = GEN_VECTOR_COUNT(table);
  if ( count >= GEN_VECTOR_SIZE(table) )
    GEN_VECTOR_ENSURE(table,count+1);

  if ( index < 0 )
    index = 0;
  else if ( index >= (GEN_INDEX)count )
    index = count;

  items = GEN_VECTOR_ITEMS(table);
  len   = (GEN_UINDEX)( count - index );
  if ( len > 0 )
    m_mem_move( (M_Byte*)items + (index + 1)*sizeof(GEN_ITEM),
                (M_Byte*)items + (index    )*sizeof(GEN_ITEM),
                len * sizeof(GEN_ITEM) );

  GEN_ITEM_SET(item,table);

  items[index]            = item;
  GEN_VECTOR_COUNT(table) = count + 1;

}

#undef GEN_VECTOR_INSERT_DEF
#undef GEN_VECTOR_INSERT
#endif



#ifdef GEN_VECTOR_ADD_DEF

#ifndef GEN_VECTOR_ADD
#define GEN_VECTOR_ADD  GEN_TABLE_FUNC(_add)
#endif

GEN_DEF( void )
GEN_VECTOR_ADD( GEN_TABLE  table,
                GEN_ITEM   item )
{
  GEN_UINDEX    count;
  GEN_ITEM*     items;

  GEN_TABLE_ASSERT(table);

  count = GEN_VECTOR_COUNT(table);
  if ( count >= GEN_VECTOR_SIZE(table) )
    GEN_VECTOR_ENSURE(table,count+1);

  items = GEN_VECTOR_ITEMS(table);

  GEN_ITEM_SET(item,table);

  items[count]            = item;
  GEN_VECTOR_COUNT(table) = count + 1;

}

#undef GEN_VECTOR_ADD_DEF
#undef GEN_VECTOR_ADD
#endif /* GEN_VECTOR_ADD_DEF */




#ifdef GEN_VECTOR_PREPEND_DEF

#ifndef GEN_VECTOR_PREPEND
#define GEN_VECTOR_PREPEND  GEN_TABLE_FUNC(_prepend)
#endif

GEN_DEF( void )
GEN_VECTOR_PREPEND( GEN_TABLE  table,
                    GEN_ITEM   item )
{
  GEN_UINDEX  count, len;
  GEN_NODE*   items;

  GEN_TABLE_ASSERT(table);

  count = GEN_VECTOR_COUNT(table);
  if ( count >= GEN_VECTOR_SIZE(table) )
    GEN_VECTOR_ENSURE(table,count+1);

  items = GEN_VECTOR_ITEMS(table);
  if ( count > 0 )
    m_mem_move( items + 1, items, count*sizeof(GEN_ITEM) );

  GEN_ITEM_SET(item,table);

  items[0]                = item;
  GEN_VECTOR_COUNT(table) = count + 1;
}

#undef GEN_VECTOR_PREPEND_DEF
#undef GEN_VECTOR_PREPEND
#endif /* GEN_VECTOR_PREPEND_DEF */



#ifdef GEN_VECTOR_REMOVE_DEF

#ifndef GEN_VECTOR_REMOVE
#define GEN_VECTOR_REMOVE  GEN_TABLE_FUNC(_remove)
#endif

GEN_DEF( void )
GEN_VECTOR_REMOVE( GEN_TABLE  table,
                   GEN_INDEX     index )
{
  GEN_INDEX  count, len;
  GEN_ITEM   item, *items;

  GEN_TABLE_ASSERT(table);

  count = (GEN_INDEX) GEN_VECTOR_COUNT(table);
  if ( (GEN_UINDEX)index >= (GEN_UINDEX)count )
    return;

  len   = count - index - 1;
  items = GEN_VECTOR_ITEMS(table);
  item  = items[index];
  GEN_ITEM_UNSET(item,table);

  if ( len > 0 )
    m_mem_move( items + index, items + index+1, len*sizeof(GEN_ITEM) );

  GEN_VECTOR_COUNT(table) = (GEN_UINDEX)(count - 1);
}

#undef GEN_VECTOR_REMOVE_DEF
#undef GEN_VECTOR_REMOVE
#endif /* GEN_VECTOR_REMOVE_DEF */



#ifdef GEN_VECTOR_ADD_SLICE_DEF

#ifndef GEN_VECTOR_ADD_SLICE
#define GEN_VECTOR_ADD_SLICE  GEN_TABLE_FUNC(_add_slice)
#endif

GEN_DEF( void )
GEN_VECTOR_ADD_SLICE( GEN_TABLE  table,
                      GEN_INDEX  item_count,
                      GEN_ITEM*  items )
{
  GEN_INDEX  count1, count2;

  GEN_TABLE_ASSERT(table);

  count1 = (GEN_INDEX) GEN_VECTOR_COUNT(table);
  count2 = item_count;
  if ( count2 > 0 )
  {
    GEN_ITEM  item, *items1, *items2;

    if ( count1 + count2 > (GEN_INDEX) GEN_VECTOR_SIZE(table) )
      GEN_VECTOR_ENSURE(table,count1+count2);

    items1 = GEN_VECTOR_ITEMS(table) + count1;
    items2 = items;
    for ( ; count2 > 0; count2--, items1++, items2++ )
    {
      item      = items2[0];
      GEN_ITEM_SET(item,table);
      items1[0] = item;
    }
  }
}

#undef GEN_VECTOR_ADD_SLICE_DEF
#undef GEN_VECTOR_ADD_SLICE
#endif /* GEN_VECTOR_ADD_SLICE_DEF */



#ifdef GEN_VECTOR_INSERT_SLICE_DEF

#ifndef GEN_VECTOR_INSERT_SLICE
#define GEN_VECTOR_INSERT_SLICE  GEN_TABLE_FUNC(_insert_slice)
#endif

GEN_DEF( void )
GEN_VECTOR_INSERT_SLICE( GEN_TABLE  table,
                         GEN_INDEX     index,
                         GEN_INDEX     item_count,
                         GEN_ITEM*  items )
{
  GEN_INDEX    count;
  GEN_ITEM  item, *items1, *items2;

  GEN_TABLE_ASSERT(table);

  if ( item_count <= 0 || items == NULL )
    return;

  count = (GEN_INDEX) GEN_VECTOR_COUNT(table);
  if ( count + item_count > (GEN_INDEX)GEN_VECTOR_SIZE(table) )
    GEN_VECTOR_ENSURE( table, count+item_count );

  if ( index < 0 )
    index = 0;
  else if ( index > count )
    index = count;

  items1 = GEN_VECTOR_ITEMS(table) + index;
  items2 = items;

  if ( index > count )
    m_mem_move( items1 + item_count,
                items1,
                (count - index)*sizeof(GEN_ITEM) );

  for ( ; item_count > 0; item_count--, items1++, items2++ )
  {
    item      = items2[0];
    GEN_ITEM_SET(item,table);
    items1[0] = item;
  }
}

#undef GEN_VECTOR_INSERT_SLICE_DEF
#undef GEN_VECTOR_INSERT_SLICE
#endif /* GEN_VECTOR_INSERT_SLICE_DEF */


#ifdef GEN_VECTOR_REMOVE_SLICE_DEF

#ifndef GEN_VECTOR_REMOVE_SLICE
#define GEN_VECTOR_REMOCE_SLICE  GEN_TABLE_FUNC(_remove_slice)
#endif

GEN_DEF( void )
GEN_VECTOR_REMOVE_SLICE( GEN_TABLE  table,
                         GEN_INDEX     index,
                         GEN_INDEX     size )
{
  GEN_INDEX     count, first, last, len;
  GEN_ITEM   item, *items;

  GEN_TABLE_ASSERT(table);

  count = (GEN_INDEX) GEN_VECTOR_COUNT(table);
  first = index;
  last  = index + size;

  /* check and correct bounds */
  if ( last <= 0 || first >= (GEN_INDEX)count || last <= first )
    return;

  if ( first < 0 )
    first = 0;

  if ( last > count )
    last = count;

  /* finalize deleted items */
  items = GEN_VECTOR_ITEMS(table);
  for ( index = first; index < last; index++ )
  {
    item         = items[index];
    items[index] = GEN_ITEM_0;
    GEN_ITEM_UNSET(item,table);
  }

  /* move trailing items */
  len = count - last;
  m_mem_move( items + first, items + last, len*sizeof(GEN_ITEM) );

  /* clear left-overs to be sure */
  items += len;
  for ( index = first; index < last; index++ )
    items[index] = GEN_ITEM_0;

  GEN_VECTOR_COUNT(table) = (GEN_UINDEX)( count - first + last );
}

#undef GEN_VECTOR_REMOVE_SLICE_DEF
#undef GEN_VECTOR_REMOCE_SLICE
#endif /* GEN_VECTOR_REMOVE_SLICE_DEF */



#ifdef GEN_VECTOR_CROP_SLICE_DEF

#ifndef GEN_VECTOR_CROP_SLICE
#define GEN_VECTOR_CROP_SLICE  GEN_TABLE_FUNC(_crop_slice)
#endif

GEN_DEF( void )
GEN_VECTOR_CROP_SLICE( GEN_TABLE  table,
                       GEN_INDEX     index,
                       GEN_INDEX     size )
{
  GEN_INDEX  count, first, last;
  GEN_ITEM   item, *items;

  GEN_TABLE_ASSERT(table);

  count = (GEN_INDEX) GEN_VECTOR_COUNT(table);
  first = index;
  last  = index + size;

  if ( first < 0 )
    first = 0;
  if ( last > count )
    last = count;

  if ( first >= last )
    first = last = 0;

  items = GEN_VECTOR_ITEMS(table);

  /* finalize items before the crop */
  if ( first > 0 )
  {
    for ( index = 0; index < first; index++ )
    {
      item         = items[index];
      items[index] = GEN_ITEM_0;
      GEN_ITEM_UNSET(item,table);
    }
  }
  /* finalize items after the crop */
  if ( last < count )
  {
    for ( index = last; index < count; index++ )
    {
      item         = items[index];
      items[index] = GEN_ITEM_0;
      GEN_ITEM_UNSET(item,table);
    }
  }

  if ( first > 0 )
    m_mem_move( items, items + first, (last-first)*sizeof(GEN_ITEM) );

  GEN_VECTOR_COUNT(table) = (GEN_UINDEX)( last - first );
}

#undef GEN_VECTOR_CROP_SLICE_DEF
#undef GEN_VECTOR_CROP_SLICE
#endif  /* GEN_VECTOR_CROP_SLICE_DEF */



#ifdef GEN_VECTOR_QSORT_DEF

#ifndef GEN_VECTOR_QSORT
#define GEN_VECTOR_QSORT  GEN_TABLE_FUNC(_quick_sort)
#endif

#ifndef GEN_ITEM_COMPARE
#error  "GEN_ITEM_COMPARE undefined in vector quick-sort template instantiation"
#endif

GEN_DEF( void )
GEN_VECTOR_QSORT( GEN_TABLE  table )
{
  GEN_ITEM*  cells;
  GEN_UINDEX    count;

  GEN_TABLE_ASSERT(table);

  cells = GEN_VECTOR_ITEMS(table);
  count = GEN_VECTOR_COUNT(table);

#define  GEN_CELL          GEN_ITEM
#define  GEN_CELL_COMPARE  GEN_ITEM_COMPARE
#define  GEN_SORT_DEF(x)   /* nothing */
#define  GEN_SORT_QUICK(x,y)    /* nothing */
#include <mlib/generic/qsort.i>

}

#undef GEN_VECTOR_QSORT_DEF
#undef GEN_VECTOR_QSORT
#endif /* GEN_VECTOR_QSORT_DEF */


#ifdef GEN_VECTOR_HEAPSORT_DEF

#ifndef GEN_VECTOR_HEAPSORT
#define GEN_VECTOR_HEAPSORT  GEN_TABLE_FUNC(_heap_sort)
#endif

#ifndef GEN_ITEM_COMPARE
#error  "GEN_ITEM_COMPARE undefined in vector heap-sort template instantiation"
#endif

GEN_DEF( void )
GEN_VECTOR_HEAPSORT( GEN_TABLE  table )
{
  GEN_ITEM*   cells;
  GEN_UINDEX  count;

  GEN_TABLE_ASSERT(table);

  cells = GEN_VECTOR_ITEMS(table);
  count = GEN_VECTOR_COUNT(table);

#define  GEN_CELL            GEN_ITEM
#define  GEN_CELL_COMPARE    GEN_ITEM_COMPARE
#define  GEN_SORT_DEF(x)     /* nothing */
#define  GEN_SORT_HEAP(x,y)  /* nothing */
#include <mlib/generic/heapsort.i>

}

#undef GEN_VECTOR_HEAPSORT_DEF
#undef GEN_VECTOR_HEAPSORT
#endif /* GEN_VECTOR_HEAPSORT_DEF */


/* undefine every template macro now */
#undef GEN_VECTOR_COUNT
#undef GEN_VECTOR_SIZE
#undef GEN_VECTOR_ITEMS
#undef GEN_VECTOR_INCREMENT

#undef GEN_VECTOR_ENSURE

#include <mlib/generic/undefs.i>
