#ifndef __MLIB_INTERNAL_BIT_SET_H__
#define __MLIB_INTERNAL_BIT_SET_H__

#include <mlib/bits.h>
#include <mlib/bitset.h>

M_BEGIN_HEADER

#define  M_BITSET_BLOCK_LEN    512
#define  M_BITSET_BLOCK_BYTES  (M_BITSET_BLOCK_LEN/8)
#define  M_BITSET_BLOCK_BITS   (M_SIZEOF_INT*8)
#define  M_BITSET_BLOCK_COUNT  (M_BITSET_BLOCK_LEN/M_BITSET_BLOCK_BITS)

#define  M_BITSET_ALL_COUNT    256
#define  M_BITSET_ALL_ZERO     ((M_UInt*)0L)
#define  M_BITSET_ALL_NUM(x)   ((M_UInt*)(M_Byte*)M_BITSET_ALL_ZERO + (M_Byte)(x))
#define  M_BITSET_ALL_SET      M_BITSET_ALL_NUM( M_BITSET_ALL_COUNT )


#define  M_BITSET_ALL_DIFF(x)   \
            ((M_ULong)((M_Byte*)(x) - (M_Byte*)M_BITSET_ALL_ZERO))

#define  M_BITSET_IS_ALL(x)    \
            (M_BITSET_ALL_DIFF(x) < M_BITSET_ALL_COUNT)

#define  M_BITSET_ALL_VAL(x)  ((M_UInt)M_BITSET_ALL_DIFF(x))

  typedef struct M_BitsetRec_
  {
    M_Memory       memory;
    M_UInt32       magic;
    M_Memory       block_pool;
    M_UInt         block_count;
    M_UInt**       blocks;

  } M_BitsetRec;


#define  M_BITSET_MAGIC   0x62736574U  /* "bset" */

#define  m_assert_is_bitset( x )   \
             m_assert( (x) && (x)->magic == M_BITSET_MAGIC )

#define  m_precheck_is_bitset( x )  \
             m_precheck( (x) && (x)->magic == M_BITSET_MAGIC );

#define  M_BITSET_PATTERN2     0x0101
#define  M_BITSET_PATTERN4     0x01010101
#define  M_BITSET_PATTERN8     0x0101010101010101

#define  m_bitset_is_pattern2(x)  ( (M_Byte)((x)>>8)  == (M_Byte)(x)   )

#define  m_bitset_is_pattern4(x)  ( (M_UInt16)((x)>>16) == (M_UInt16)(x) && \
                                    m_bitset_is_pattern2(x) )

#define  m_bitset_is_pattern8(x)  ( (M_UInt32)((x)>>32) == (M_UInt32)(x) && \
                                     m_bitset_is_pattern4(x) )
#if   M_SIZEOF_INT == 2

#  define  M_BITSET_PATTERN         M_BITSET_PATTERN2
#  define  m_bitset_is_pattern(x)   m_bitset_is_pattern2( (M_UInt)(x) )

#elif M_SIZEOF_INT == 4

#  define  M_BITSET_PATTERN         M_BITSET_PATTERN4
#  define  m_bitset_is_pattern(x)   m_bitset_is_pattern4( (M_UInt)(x) )

#elif M_SIZEOF_INT == 8

#  define  M_BITSET_PATTERN         M_BITSET_PATTERN8
#  define  m_bitset_is_pattern(x)   m_bitset_is_pattern8( (M_UInt)(x) )

#else
#  error  "invalid integer bytes size"
#endif

#define  m_bitset_block_new( _bs )   \
        m_mem_alloc( M_BITSET_BLOCK_BYTES, (_bs)->block_pool )

#define  m_bitset_block_destroy( _bs, _bl )  \
        m_mem_free( _bl, (_bs)->block_pool )

#define  m_bitset_memory(s)  ((s)->memory)

  MLIB_LOCAL(M_UInt*)
  m_bitset_block_copy( M_Bitset  bitset, M_UInt*  source );

  MLIB_LOCAL(M_UInt**)
  m_bitset_set_prepare( M_Bitset  bitset,
                        M_UInt    index,
                        M_UInt    size,
                        M_UInt    value );


M_END_HEADER

#endif /* __MLIB_INTERNAL_BIT_SET_H__ */
