#ifndef __MLIB_INTERNAL_HASH_H__
#define __MLIB_INTERNAL_HASH_H__

#include <mlib/hash.h>

M_BEGIN_HEADER

#define  M_HASH_TABLE_MAGIC         0x32eda123U

  typedef struct M_HashNodeRec_*   M_HashNode;

  typedef struct M_HashNodeRec_
  {
    M_HashNode   link;
    M_ULong      hash;   /* cached hash value for faster resizes */
    M_Pointer    key;
    M_Pointer    value;

  } M_HashNodeRec;



  typedef struct M_HashTableRec_
  {
    M_Memory            memory;
    M_Memory            node_pool;
    M_UInt32            magic;
    M_ULong             size;
    M_ULong             nodes;
    M_HashNode*         buckets;

    M_HashDataFunc      key_hash;
    M_CompareDataFunc   key_compare;
    M_DestroyDataFunc   key_destroy;
    M_Pointer           key_data;

    M_DestroyDataFunc   val_destroy;
    M_Pointer           val_data;

  } M_HashTableRec;



#define  m_assert_is_hash_table(x)     \
            m_assert( (x) && (x)->magic == M_HASH_TABLE_MAGIC )

#define  M_HASH_KEY_HASH(key,table)                                       \
             ((table->key_hash) ? table->key_hash( key, table->key_data ) \
                                : M_POINTER_TO_DIST(key))

/* forward declarations */
  MLIB_LOCAL(M_HashNode)
  m_hash_node_new( M_HashTable, M_Pointer, M_Pointer );

  MLIB_LOCAL(void)
  m_hash_node_destroy( M_HashNode, M_HashTable );

  MLIB_LOCAL(void)
  m_hash_table_resize( M_HashTable  table );

  MLIB_LOCAL(void)
  m_hash_table_clear( M_HashTable  table );

  MLIB_LOCAL( M_HashNode* )
  m_hash_table_get_nodep( M_HashTable, M_Pointer );


 /* */

M_END_HEADER

#endif /* __MLIB_INTERNAL_HASH_H__ */

