#ifndef __MLIB_SORT_H__
#define __MLIB_SORT_H__

#include <mlib.h>

M_BEGIN_HEADER

 /***************************************************************
  *
  * @function: m_heap_sort_ptrs
  *
  * @description:
  *   perform in-place sorting of an array using the heap-sort
  *   algorithm.
  *
  * @input:
  *   table        :: target array
  *   count        :: number of elements in array
  *   compare_func :: comparison function
  *   compare_data :: comparison data
  *
  * @note:
  *   this function will _only_ work on pointer arrays.
  */
  MLIB_API(void)   m_heap_sort_ptrs( M_Pointer*     table,
                                     M_UFast        count,
                                     M_CompareFunc  compare );


 /***************************************************************
  *
  * @function: m_heap_sort
  *
  * @description:
  *   perform in-place sorting of an array using the heap-sort
  *   algorithm.
  *
  * @input:
  *   table        :: target array
  *   count        :: number of elements in array
  *   compare_func :: comparison function
  *   compare_data :: comparison data
  *
  * @note:
  *   this function will _only_ work on pointer arrays.
  */
  MLIB_API(void)   m_heap_sort( M_Pointer      table,
                                M_UFast        item_count,
                                M_Size         item_size,
                                M_CompareFunc  compare );


 /***************************************************************
  *
  * @function: m_quick_sort_ptrs
  *
  * @description:
  *   perform in-place sorting of a pointer array using the
  *   quick-sort algorithm
  *
  * @input:
  *   table   :: target array
  *   count   :: number of elements in array
  *   compare :: comparison function
  *
  * @note:
  *   this function will _only_ work on pointer arrays. use
  *   the @m_quick_sort if you need to sort arbitrary structures,
  *   or instantiate the quicksort template file in your own
  *   code
  */
  MLIB_API(void)   m_quick_sort_ptrs( M_Pointer*     table,
                                      M_UFast        count,
                                      M_CompareFunc  compare );


 /***************************************************************
  *
  * @function: m_quick_sort
  *
  * @description:
  *   perform in-place sorting of an array of any item type
  *   on most platform, this simply calls then ANSI C 'qsort'
  *
  * @input:
  *   table        :: array address
  *   item_size    :: size of item in bytes
  *   item_count   :: number of items
  *   compare_func :: comparison function
  *   compare_data :: comparison data
  *
  * @note:
  *   if you need to sort pointer tables, consider using the functions
  *   @m_quick_sort_ptrs or @m_heap_sort_ptrs which should be faster
  */
  MLIB_API(void)  m_quick_sort( M_Pointer      array,
                                M_UFast        count,
                                M_Size         width,
                                M_CompareFunc  compare );

 /* */

M_END_HEADER

#endif /* __MLIB_SORT_H__ */
