#ifndef __MLIB_TESTS_H__
#define __MLIB_TESTS_H__

#ifndef __MLIB_H__
#error "do not include this file directly, use <mlib.h> instead"
#endif

M_BEGIN_HEADER

  typedef struct M_TestRec_*               M_Test;

  typedef const struct M_Test_ClassRec_*   M_Test_Class;

  typedef struct M_TestRec_
  {
    M_Core        mcore;
    M_Memory      memory;
    M_Test_Class  clazz;
    M_Bool        passed;
    M_UInt        log_level;
    M_UInt        subtest;

  } M_TestRec;

  typedef void  (*M_Test_RunFunc)( M_Test   test );

  typedef struct M_Test_ClassRec_
  {
    M_CString        test_name;
    M_Size           test_size;
    M_Test_RunFunc   test_run;

  } M_Test_ClassRec;

  MLIB_API( void )
  m_core_add_test( M_Core         mcore,
                   M_Test_Class   clazz );

  MLIB_API( M_Bool )
  m_core_run_tests( M_Core      mcore,
                    M_UInt      log_level,
                    M_UInt     *anum_success,
                    M_UInt     *anum_failures );

  MLIB_API( void )
  m_test_reset( M_Test  test );

  MLIB_API( void )
  m_test_log( const char*  fmt, ... );

#define  M_TEST_BEGIN                \
          {                          \
            m_test_reset( test );    \
            do {

#define  M_TEST_END                  \
            } while (0);             \
          }

#define  M_TEST_LOG(n,fmt)             \
    M_BEGIN_STMNT                      \
      if ( test->log_level <= (n) )    \
        m_test_log  fmt ;              \
    M_END_STMNT

#define  M_TEST_LOG0(fmt)  M_TEST_LOG(0,fmt)
#define  M_TEST_LOG1(fmt)  M_TEST_LOG(1,fmt)
#define  M_TEST_LOG2(fmt)  M_TEST_LOG(2,fmt)

#define  M_TEST_ASSERT(cond)                             \
    M_BEGIN_STMNT                                        \
      if ( ! (cond) )                                    \
      {                                                  \
        m_test_log ( "assertion failed (%s:%d): %s\n",   \
                     __FILE__, __LINE__, #cond );        \
                                                         \
        test->passed = 0;                                \
        break;                                           \
      }                                                  \
    M_END_STMNT

 /* */

  MLIB_API( void )
  m_core_add_test_classes( M_Core         mcore,
                           M_Test_Class*  list );

#ifdef M_CONFIG_OPTION_UNIT_TESTS
#  define  MLIB_TEST(component)     MLIB_VAR( M_Test_Class )    m_ ## component ## __test_classes[] ;
#  define  MLIB_TEST_DEF(function)    & function ## __test_class ,
#else
#  define  MLIB_TEST(component)     /* nothing */
#  define  MLIB_TEST_DEF(function)   /* nothing */
#endif

 /* */

M_END_HEADER

#endif /* __MLIB_TESTS_H__ */
