#include <mlib/internal/ibitset.h>

  MLIB_APIDEF(void)
  m_bitset_set_range( M_Bitset  bitset,
                      M_UInt    start,
                      M_UInt    count,
                      M_UInt    size,
                      M_UInt    value )
  {
    m_assert( M_BITS_IS_POWER2(size) && size && size < 16 );

    if ( bitset && count && M_BITS_IS_POWER2(size) && size < 16 )
    {
      M_UInt  end;

      start *= size;
      count *= size;
      end    = start + count;

      /* create pattern from value */
      value &= (1 << size) - 1;
      while (size < 8)
      {
        value |= (value << size);
        size  *= 2;
      }
      value *= M_BITSET_PATTERN;

      /* grow bitset when needed */
      if ( end > bitset->block_count * M_BITSET_BLOCK_LEN )
      {
        if ( value )
          m_bitset_ensure( bitset, end );
        else
          count = bitset->block_count * M_BITSET_BLOCK_LEN - start;
      }

      while ( count > 0 )
      {
        M_UInt    delta, pos;
        M_UInt**  pblock;
        M_UInt*   block;

        pos   = start % M_BITSET_BLOCK_LEN;
        delta = M_BITSET_BLOCK_LEN - pos;
        if (delta > count)
          delta = count;

        pblock = bitset->blocks + (start / M_BITSET_BLOCK_LEN);
        block  = pblock[0];

        if (delta == M_BITSET_BLOCK_LEN)
        {
          if ( !M_BITSET_IS_ALL(block) )
            m_bitset_block_destroy( bitset, block );

          pblock[0] = M_BITSET_ALL_NUM(value);
        }
        else
        {
          M_UInt  mask1, mask2, len;

          if ( M_BITSET_IS_ALL(block) )
          {
            M_UInt  val = M_BITSET_ALL_VAL(block);

            if ( val == (M_UInt)(M_Byte)value )
              goto Next;

            block = pblock[0] = m_bitset_block_new( bitset );
            M_MEM_SET( block, (M_Byte)val, M_BITSET_BLOCK_BYTES );
          }

          end    = pos + delta - 1;
          block += pos / M_BITSET_BLOCK_BITS;
          len    = ( end / M_BITSET_BLOCK_BITS ) - ( pos / M_BITSET_BLOCK_BITS );
          mask1  = (M_UInt) (-1L << ( pos % M_BITSET_BLOCK_BITS ));
          mask2  = (M_UInt)~(-1L << ( 1 + ( end % M_BITSET_BLOCK_BITS ) ) );

          if ( len > 0 )
          {
            block[0] = (block[0] & ~mask1) | (value & mask1);

            for ( block++, len--; len > 0; len--, block++ )
              block[0] = value;

            block[0] = (block[0] & ~mask2) | (value & mask2);
          }
          else
          {
            mask1   &= mask2;
            block[0] = (block[0] & ~mask1) | (value & mask1);
          }
        }

      Next:
        count -= delta;
        start += delta;
      }
    }
  }


