#include <mlib/internal/ibitset.h>

  MLIB_APIDEF(M_Bitset)
  m_bitset_union( M_Bitset  set1,
                  M_Bitset  set2,
                  M_Bool    push )
  {
    M_UInt    count, **pblock1, **pblock2, **pblock;
    M_Bitset  bitset;

    if ( !set1 )
      return m_bitset_copy( set2, push );

    if ( !set2 )
      return m_bitset_copy( set1, push );

    bitset = m_bitset_new( set1->memory, 1 );
    count  = M_MAX( set1->block_count, set2->block_count );

    m_bitset_ensure( bitset, count * M_BITSET_BLOCK_LEN );

    /* compute union */

    pblock1 = set1->blocks;
    pblock2 = set2->blocks;
    pblock  = bitset->blocks;

    count  = M_MIN( set1->block_count, set2->block_count );
    for ( ; count > 0; count--, pblock1++, pblock2++, pblock++ )
    {
      M_UInt   count2 = M_BITSET_BLOCK_COUNT;
      M_UInt*  block1 = pblock1[0];
      M_UInt*  block2 = pblock2[0];
      M_UInt*  block;
      M_UInt   val1, val2;

      if ( M_BITSET_IS_ALL(block1) )
      {
        val1 = M_BITSET_ALL_VAL(block1);
        if (!val1)
        {
          block = m_bitset_block_copy( bitset, block2 );
        }
        else if ( M_BITSET_IS_ALL(block2))
        {
          val2 = M_BITSET_ALL_VAL(block2);
          block = M_BITSET_ALL_NUM( val1 | val2 );
        }
        else
        {
          M_UInt*  p;

          block = p = m_bitset_block_new( bitset );
          for ( ; count2 > 0; count2--, p++, block2++ )
            p[0] = ( val1 | block2[0] );
        }
      }
      else if ( M_BITSET_IS_ALL(block2) )
      {
        val2 = M_BITSET_ALL_VAL(block2);
        if (!val2)
          block = m_bitset_block_copy( bitset, block1 );
        else
        {
          M_UInt*  p;

          block = p = m_bitset_block_new( bitset );
          for ( ; count2 > 0; count2--, p++, block1++ )
            p[0] = ( block1[0] | val2 );
        }
      }
      else
      {
        M_UInt*  p;

        block = p = m_bitset_block_new( bitset );
        for ( ; count2 > 0; count2--, p++, block1++, block2++ )
          p[0] = ( block1[0] | block2[0] );
      }

      pblock[0] = block;
    }

    /* now copy remaining bit blocks */
    if ( set1->block_count > set2->block_count )
    {
      count = set1->block_count - set2->block_count;
    }
    else
    {
      count   = set2->block_count - set1->block_count;
      pblock1 = pblock2;
    }

    for ( ; count > 0; count--, pblock++, pblock1++ )
      pblock[0] = m_bitset_block_copy( bitset, pblock1[0] );

    /* compress result */
    m_bitset_compress( bitset );

    if (!push)
      m_bitset_pop( bitset, TRUE );

    return bitset;
  }


