#include <mlib/internal/icalc.h>

#ifdef M_CONFIG_HAS_INT64

  MLIB_APIDEF(M_Fix16)
  m_fix16_div( M_Fix16   a,
               M_Fix16   b )
  {
    M_Fix16  s;
    M_Int64  val;

    s = a^b;
    if (a < 0) a = -a;
    if (b < 0) b = -b;

    val = M_INT64(a) << 32;

    if ( val >= (M_INT64(b) << 32) )
      val = M_FIX16_MAX;
    else
      val /= b;

    return M_FIX16( (s >= 0) ? val : -val );
  }

#else /* !M_CONFIG_HAS_INT64 */

  MLIB_APIDEF(M_Fix16)
  m_fix16_div( M_Fix16   a,
               M_Fix16   b )
  {
    M_Int32   s;
    M_UInt32  q;


    s  = a^b;
    if (a < 0) a = -a;
    if (b < 0) b = -b;

    if ( b == 0 )
    {
      /* check for division by 0 */
      q = M_FIX16_MAX;
    }
    else if ( ( a >> 16 ) == 0 )
    {
      /* compute result directly when possible */
      q = M_UINT32(a << 16)/M_UINT32(b);
    }
    else
    {
      /* we need more bits */
      M_Card64Rec  z;

      m_card64_mul( &z, M_UINT32(a), M_UINT32_C(0x10000) );

      q = m_card64_div( &z, M_UINT32(b) );

      if ( q > M_FIX16_MAX )
        q = M_FIX16_MAX;
    }

    return M_FIX16( (s < 0) ? -q : q );
  }

#endif /* !M_CONFIG_HAS_INT64 */


