#include <mlib/internal/icalc.h>

#ifdef M_CONFIG_HAS_INT64

  MLIB_APIDEF(M_Fix16)
  m_fix16_mul_round( M_Fix16   a,
                     M_Fix16   b )
  {
    M_Fix16  s;

    M_Int64  val;

    s = a^b;
    if (a < 0) a = -a;
    if (b < 0) b = -b;

    val = ( M_INT64(a) * b +
            M_UINT32_C(0x80000000) ) >> 32;

    if ( val > M_FIX16_MAX )
      val = M_FIX16_MAX;

    return M_FIX16( s >= 0 ? val : -val );
  }

#else /* !M_CONFIG_HAS_INT64 */

  MLIB_APIDEF(M_Fix16)
  m_fix16_mul_round( M_Fix16  a,
                     M_Fix16  b )
  {
    M_Long    s;
    M_UInt32  ua, ub;


    if ( a == 0 || b == M_INT32_C(0x10000) )
      return a;

    s  = a^b;
    if ( a < 0 ) a = -a;
    if ( b < 0 ) b = -b;

    ua = M_UINT32(a);
    ub = M_UINT32(b);

    /* this optimisation comes from FreeType 2 */
    if ( ua <= 2048 && ub <= 1048576L )
    {
      /* the product fits in 32-bits */
      ua = ( ua * ub + 0x8000 ) >> 16;
    }
    else
    {
      M_UInt32  al = ua & 0xFFFF;

      /* XXXX: need to add rounding term here !! */
      ua = ( ua >> 16 ) * ub +
           al * ( ub >> 16 ) +
           ( al * ( ub & 0xFFFF ) >> 16 );
    }
    return M_FIX16( s < 0 ? -ua : ua );
  }

#endif /* !M_CONFIG_HAS_INT64 */

