#include <mlib.h>
#include <mlib/internal/icore.h>


  static const M_ExceptionRec  m_err_lock_init_rec =
  {
    &mlib_exception_family,
    M_ERR_LOCK_INIT
  };



  static const M_ExceptionRec  m_err_lock_enter_rec =
  {
    &mlib_exception_family,
    M_ERR_LOCK_ENTER
  };


  static const M_ExceptionRec  m_err_lock_exit_rec =
  {
    &mlib_exception_family,
    M_ERR_LOCK_EXIT
  };

  MLIB_VARDEF( M_Exception )  m_err_lock_init  = &m_err_lock_init_rec;
  MLIB_VARDEF( M_Exception )  m_err_lock_enter = &m_err_lock_enter_rec;
  MLIB_VARDEF( M_Exception )  m_err_lock_exit  = &m_err_lock_exit_rec;


  MLIB_APIDEF( M_Lock )
  m_lock_new( M_Core      mcore,
              M_LockType  type )
  {
#ifdef M_CONFIG_TRHEAD_SAFE

    M_Lock_ClassRec*  clazz = NULL;
    M_Memory          memory;
    M_Lock            lock;
    M_ICore           icore;

    m_assert_is_core( mcore );

    icore  = M_ICORE(mcore);
    memory = mcore->memory;

    switch (type)
    {
      case M_LOCK_MUTEX:      clazz = &icore->mutex_class;    break;
      case M_LOCK_SPIN:       clazz = &icore->spinlock_class; break;
      case M_LOCK_READWRITE:  clazz = &icore->rwlock_class;   break;
      default:
        m_assert_unreached;
    }

    lock = M_ALLOC( clazz->size );
    if ( clazz->init( lock, type, 0, NULL ) )
    {
      M_FREE( lock );
      m_core_throw( mcore, m_err_lock_init );
    }

    return lock;

#else /* !M_CONFIG_THREAD_SAFE */
    m_assert_is_core( mcore );

    M_UNUSED( mcore );
    M_UNUSED( type );

    return NULL;
#endif /* !M_CONFIG_THREAD_SAFE */
  }



  MLIB_APIDEF( void )
  m_lock_push( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock( lock );
      clazz = lock->clazz;

      m_core_push( clazz->mcore, lock, (M_CleanupFunc)clazz->exit, NULL );
    }
  }


  MLIB_APIDEF( void )
  m_lock_pop( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock( lock );
      clazz = lock->clazz;

      m_core_pop( clazz->mcore, lock, M_POP_KEEP );
    }
  }


  MLIB_APIDEF( void )
  m_lock_destroy( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;
      M_Memory      memory;

      m_assert_is_lock( lock );

      clazz  = lock->clazz;
      memory = clazz->mcore->memory;

      if ( clazz->done )
        clazz->done( lock );

      lock->clazz = NULL;
      M_FREE( lock );
    }
  }


  MLIB_APIDEF( void )
  m_lock_enter( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock(lock);

      clazz = lock->clazz;

      if ( clazz->enter( lock ) )
        m_core_throw( clazz->mcore, m_err_lock_enter );
    }
  }


  MLIB_APIDEF( void )
  m_lock_exit( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock(lock);

      clazz = lock->clazz;

      if ( clazz->exit( lock ) )
        m_core_throw( clazz->mcore, m_err_lock_exit );
    }
  }


  MLIB_APIDEF( void )
  m_lock_enter_read( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock(lock);

      clazz = lock->clazz;

      if ( clazz->enter_read( lock ) )
        m_core_throw( clazz->mcore, m_err_lock_enter );
    }
  }


  MLIB_APIDEF( void )
  m_lock_enter_update( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock(lock);

      clazz = lock->clazz;

      if ( clazz->enter_update( lock ) )
        m_core_throw( clazz->mcore, m_err_lock_enter );
    }
  }


  MLIB_APIDEF( void )
  m_lock_write_update( M_Lock  lock )
  {
    if ( lock )
    {
      M_Lock_Class  clazz;

      m_assert_is_lock(lock);

      clazz = lock->clazz;

      if ( clazz->write_update( lock ) )
        m_core_throw( clazz->mcore, m_err_lock_enter );
    }
  }

