#include <mlib.h>
#include <mlib/internal/itests.h>
#include <stdarg.h>
#include <stdio.h>

 /***************************************************************************/
 /***************************************************************************/
 /*****                                                                 *****/
 /*****                 T E S T   S T A T I C   O B J E C T             *****/
 /*****                                                                 *****/
 /***************************************************************************/
 /***************************************************************************/

  static void
  m_test_static_done( M_TestStatic  statics )
  {
    M_Memory  memory = statics->memory;

    /* remove all test nodes */
    {
      M_TestNode  node, next;

      node = statics->tests;
      while ( node )
      {
        next = node->next;
        M_FREE( node );
        node = next;
      }
      statics->tests      = NULL;
      statics->tests_tail = &statics->tests;
    }
  }

  static void
  m_test_static_init( M_TestStatic  statics,
                       M_Core        mcore )
  {
    statics->mcore  = mcore;
    statics->memory = mcore->memory;

    statics->tests      = NULL;
    statics->tests_tail = &statics->tests;

    statics->num_success = 0;
    statics->num_failed  = 0;
  }

#define  M_TEST_STATIC_ID   0x74657374   /* "test" */

  static const M_Static_ClassRec   m_test_static_class =
  {
    sizeof( M_TestStaticRec ),
    M_TEST_STATIC_ID,
    (M_Static_InitFunc)  m_test_static_init,
    (M_Static_DoneFunc)  m_test_static_done
  };

 /***************************************************************************/
 /***************************************************************************/
 /*****                                                                 *****/
 /*****                 T E S T   R O U T I N E S                       *****/
 /*****                                                                 *****/
 /***************************************************************************/
 /***************************************************************************/

  static  M_Test  mlib_current_test = NULL;

  MLIB_APIDEF( void )
  m_core_add_test( M_Core         mcore,
                   M_Test_Class   clazz )
  {
    M_Memory      memory  = mcore->memory;
    M_TestStatic  statics = m_core_get_static( mcore, &m_test_static_class, NULL );
    M_TestNode    node;

    M_NEW( node );
    node->clazz  = clazz;
    node->passed = 0;
    node->next   = NULL;

    /* append at the end of the list */
    *statics->tests_tail = node;
    statics->tests_tail  = &node->next;
  }


  MLIB_APIDEF( M_Bool )
  m_core_run_tests( M_Core      mcore,
                    M_UInt      log_level,
                    M_UInt     *anum_success,
                    M_UInt     *anum_failures )
  {
    M_Memory      memory  = mcore->memory;
    M_TestStatic  statics = m_core_get_static( mcore, &m_test_static_class, NULL );
    M_TestNode volatile   node;
    M_Test volatile       test;

    statics->num_success = 0;
    statics->num_failed  = 0;

    node = statics->tests;
    while ( node )
    {
      test            = M_ALLOC( node->clazz->test_size );
      test->memory    = memory;
      test->mcore     = mcore;
      test->clazz     = node->clazz;
      test->subtest   = 0;
      test->passed    = 1;  /* by default !! */
      test->log_level = log_level;

      mlib_current_test = test;

      M_XTRY
      {
        node->clazz->test_run( test );
      }
      M_XCATCH_ANY
      {
        /* an exception was raised in the test */
        test->passed = 0;

        m_test_log( "unexpected exception thrown !!\n" );
      }
      M_XEND

      mlib_current_test = NULL;

      if ( test->passed != 0 )
        statics->num_success ++;
      else
        statics->num_failed ++;

      M_FREE( test );

      node = node->next;
    }

    *anum_success  = statics->num_success;
    *anum_failures = statics->num_failed;

    return ( statics->num_failed != 0 );
  }


  MLIB_APIDEF( void )
  m_test_reset( M_Test  test )
  {
    test->subtest++;

    if ( test->subtest == 1 )
      m_test_log( "testing\n" );
  }


 /* XXX: for now, we send the output to stderr, can we change that ?? */
  MLIB_APIDEF( void )
  m_test_log( const char*  fmt, ... )
  {
    va_list  ap;

    printf( "  %s: ", mlib_current_test->clazz->test_name );

    va_start( ap, fmt );
    vprintf( fmt, ap );
    va_end( ap );
  }



  MLIB_APIDEF( void )
  m_core_add_test_classes( M_Core         mcore,
                           M_Test_Class*  list )
  {
    M_Test_Class*  cnode = list;

    while ( *cnode )
    {
      m_core_add_test( mcore, *cnode );
      cnode++;
    }
  }

