#include <mlib/internal/imemory.h>

  MLIB_APIDEF(void)
  m_mem_realloc( M_Pointer *apointer,
                 M_Size     cur_size,
                 M_Size     new_size,
                 M_Memory   memory )

  {
    M_Pointer  p;
    M_IMemory  internal = M_IMEMORY(memory);
    M_Error    error;

    m_assert_is_memory( memory );
    m_assert( apointer );

    error = 0;
    p     = *apointer;

#ifdef MLIB_CONFIG_OPTION_MEMORY_POOLS
    if ( internal->pool )
      M_PANIC(( "%s: trying to reallocate fixed-size memory block\n",
                "mem.realloc" ));
#endif

    if (cur_size == 0)
    {
      /* allocate new block */
      *apointer = m_mem_alloc( new_size, memory );
    }
    else if (new_size == 0)
    {
      /* free current block */
      if (p)
      {
        internal->free( p, memory );
        *apointer = 0;
      }
    }
    else
    {
      M_Pointer  q;

      /* reallocate block */
      q = internal->realloc( p, new_size, cur_size, memory );
      if (q)
      {
        *apointer = q;

        /* clear memory block's new content if needed */
        if ( m_memory_need_block_clear( memory ) && new_size > cur_size )
          M_MEM_SET( (char*)q + cur_size, 0, new_size - cur_size );
      }
      else
      {
        if ( m_memory_trap_enabled( memory ) && memory->mcore )
          m_core_throw( memory->mcore, m_err_memory_alloc );

        /* Panic otherwise */
        M_PANIC(( "%s: out of memory !!\n",
                  "mem.realloc" ));
      }
    }
  }


