#include <mlib/string.h>

  MLIB_APIDEF(M_Int)
  m_str_compare( M_CString  s1,
                 M_CString  s2 )
  {
    static const M_CString  empty = "";

    M_CString  p1 = s1;
    M_CString  p2 = s2;
    M_Int      result;

    /* a NULL string always equals an empty one */
    if ( p1 == NULL )
      p1 = empty;

    if ( p2 == NULL )
      p2 = empty;

    /* perform the comparison */
    result = 0;
    for (;;)
    {
      M_Int  c1, c2;

      c1 = *p1++;
      c2 = *p2++;

      if ( c1 != c2 )
      {
        result = c1 - c2;
        break;
      }
      if ( c1 == 0 )
      {
        result = 0;
        break;
      }
    }
    return result;
  }


#ifdef M_CONFIG_OPTION_UNIT_TESTS

  static void
  m_str_compare__test( M_Test  test )
  {
    M_TEST_BEGIN
    {
      M_TEST_ASSERT( m_str_compare( NULL, NULL ) == 0 );
      M_TEST_ASSERT( m_str_compare( NULL, "" )   == 0 );
      M_TEST_ASSERT( m_str_compare( "", NULL )   == 0 );
      M_TEST_ASSERT( m_str_compare( "", "" )     == 0 );

      M_TEST_ASSERT( m_str_compare( NULL, "a" ) < 0 );
      M_TEST_ASSERT( m_str_compare( "", "a" )   < 0 );
      M_TEST_ASSERT( m_str_compare( "a", NULL ) > 0 );
      M_TEST_ASSERT( m_str_compare( "a", "" )   > 0 );

      M_TEST_ASSERT( m_str_compare( "a", "a" ) == 0 );
      M_TEST_ASSERT( m_str_compare( "a", "A" )  > 0 );
      M_TEST_ASSERT( m_str_compare( "A", "a" )  < 0 );
      M_TEST_ASSERT( m_str_compare( "a", "z" )  < 0 );
      M_TEST_ASSERT( m_str_compare( "z", "a" )  > 0 );

      M_TEST_ASSERT( m_str_compare( "a", "ab" ) < 0 );
      M_TEST_ASSERT( m_str_compare( "ab", "a" ) > 0 );

      M_TEST_ASSERT( m_str_compare( "a", "bc" ) < 0 );
      M_TEST_ASSERT( m_str_compare( "bc", "a" ) > 0 );

      M_TEST_ASSERT( m_str_compare( "ab", "cb" ) < 0 );
      M_TEST_ASSERT( m_str_compare( "cb", "ab" ) > 0 );
    }
    M_TEST_END
  }

#define  M_TEST_FUNCTION  m_str_compare__test
#define  M_TEST_NAME      "m_str_compare"
#include <mlib/tests-def.h>

#endif /* M_CONFIG_OPTION_UNIT_TESTS */

 