#include <mlib/string.h>

  MLIB_APIDEF(M_String)
  m_str_dup( M_CString   source,
             M_Memory    memory,
             M_Bool      push )
  {
    M_String  copy = NULL;
    M_UInt    len;

    m_assert_is_memory( memory );

    if (source)
    {
      len  = m_str_len( source );
      copy = m_str_new( len, memory, push );

      M_MEM_COPY( copy, source, len+1 );
    }
    return copy;
  }


#ifdef M_CONFIG_OPTION_UNIT_TESTS

  static void
  m_str_dup__test( M_Test  test )
  {
    M_Memory  memory = test->memory;

    M_TEST_BEGIN
    {
      M_String  source, copy;

      M_TEST_ASSERT( m_str_dup( NULL, memory, 0 ) == 0 );
      M_TEST_ASSERT( m_str_dup( NULL, memory, 1 ) == 0 );

      copy = m_str_dup( "", memory, 0 );
      M_TEST_ASSERT( copy != NULL && copy[0] == 0 );

      source = "a";
      copy   = m_str_dup( source, memory, 0 );
      M_TEST_ASSERT( copy != NULL && copy != source && copy[0] == 'a' && copy[1] == 0 );

      M_FREE( copy );

      source = "abcdef";
      copy   = m_str_dup( source, memory, 0 );
      M_TEST_ASSERT( copy != NULL && copy != source );
      {
        M_CString  p1 = source;
        M_CString  p2 = copy;

        while ( *p1 )
        {
          M_TEST_ASSERT( *p1 == *p2 );
          p1++;
          p2++;
        }
      }
      M_FREE( copy );
    }
    M_TEST_END
  }

#define  M_TEST_FUNCTION  m_str_dup__test
#define  M_TEST_NAME      "m_str_dup"
#include <mlib/tests-def.h>

#endif /* M_CONFIG_OPTION_UNIT_TESTS */

