#include <mlib/string.h>

  MLIB_APIDEF( M_Bool )
  m_str_equal( M_CString  s1,
               M_CString  s2 )
  {
    static const M_CString  empty = "";

    M_CString  p1 = s1;
    M_CString  p2 = s2;
    M_Int      c1, c2;
    M_Bool     result;

    /* a NULL string always equals an empty one */
    if ( p1 == NULL )
      p1 = empty;

    if ( p2 == NULL )
      p2 = empty;

    /* perform the comparison */
    for (;;)
    {
      c1     = *p1++;
      c2     = *p2++;
      result = (c1 == c2);
      if ( !result || c1 == 0 )
        break;

      c1     = *p1++;
      c2     = *p2++;
      result = (c1 == c2);
      if ( !result || c1 == 0 )
        break;

      c1     = *p1++;
      c2     = *p2++;
      result = (c1 == c2);
      if ( !result || c1 == 0 )
        break;

      c1     = *p1++;
      c2     = *p2++;
      result = (c1 == c2);
      if ( !result || c1 == 0 )
        break;
    }

    return result;
  }

#ifdef M_CONFIG_OPTION_UNIT_TESTS

  static void
  m_str_equal__test( M_Test  test )
  {
    M_TEST_BEGIN
    {
      M_TEST_ASSERT( m_str_equal( NULL, NULL ) != 0 );
      M_TEST_ASSERT( m_str_equal( NULL, "" )   != 0 );
      M_TEST_ASSERT( m_str_equal( "", NULL )   != 0 );

      M_TEST_ASSERT( m_str_equal( NULL, "a" )  == 0 );
      M_TEST_ASSERT( m_str_equal( "a", NULL )  == 0 );
      M_TEST_ASSERT( m_str_equal( "", "a" )    == 0 );
      M_TEST_ASSERT( m_str_equal( "a", "" )    == 0 );

      M_TEST_ASSERT( m_str_equal( "a", "a" )   != 0 );
      M_TEST_ASSERT( m_str_equal( "A", "A" )   != 0 );
      M_TEST_ASSERT( m_str_equal( "a", "A" )   == 0 );
      M_TEST_ASSERT( m_str_equal( "A", "a" )   == 0 );

      M_TEST_ASSERT( m_str_equal( "cool", "cool" )  != 0 );
      M_TEST_ASSERT( m_str_equal( "cool", "cool2" ) == 0 );
      M_TEST_ASSERT( m_str_equal( "cool", "hot" )   == 0 );
    }
    M_TEST_END
  }

#define  M_TEST_FUNCTION  m_str_equal__test
#define  M_TEST_NAME      "m_str_equal"
#include <mlib/tests-def.h>

#endif /* M_CONFIG_OPTION_UNIT_TESTS */

 