#include <mlib/internal/itime.h>

  MLIB_LOCALDEF(void)
  m_itime_get_date( M_Time        time_,
                    M_DateDay    *pday,
                    M_DateMonth  *pmonth,
                    M_DateYear   *pyear,
                    M_Int32      *pseconds )
  {
    M_Int32      sec_hi;
    M_UInt32     sec_lo;
    M_Int32      day;
    M_DateRec    date;
    M_Calc64Rec  t;

   /* now, compute the number of days this corresponds to. There are */
   /* exactly 60*60*24 = 86400 seconds per day..                     */
   /*                                                                */
   t   = *M_ITIME(time_);
   day = m_calc64_div( &t, M_INT32_C(86400)*M_USER_PER_SEC );

   if ( day < 0 )
   {

   }

   m_date_set( &date, 1, 1, 1970 );
   m_date_add_days( &date, day );

   /* now, the remaining seconds are in

   /*   seconds  = hi*2^32 + lo                                      */
   /*   hi/86400 = h1 + (h2//86400)          "//" denotes floating   */
   /*   lo/86400 = l1 + (l2//86400)          point division          */
   /*                                                                */
   /*   result   = seconds//86400                                    */
   /*            = (h1 + (h2//86400))*2^32 + (l1 + (l2//86400))      */
   /*            = h1*2^32 + h2*(2^32//86400) + l1 + (l2//86400)     */
   /*                                                                */
   /*   we assume that h1 == 0, otherwise the number is simply       */
   /*   too big for us..                                             */
   /*                                                                */
   /*   2^32//86400 = 49710 + (23296//86400)                         */
   /*   result      = h2*49710 + (h2*23296//86400) +l1 + (l2//86400) */
   /*                                                                */
   /*   since h2 < 86400, it needs less than 17 bits, and            */
   /*   (h2*23296) needs less than 32 bits and can be computed in    */
   /*   a M_UInt32                                                   */
   /*                                                                */
   {
     M_UInt32  h2, l1, l2, h3;
     M_Int     sign = 0;

     /* use absolute values */
     if ( sec_hi < 0 )
     {
       if ( sec_lo )
       {
         sec_lo = (M_UInt32)(~sec_lo+1);
         sec_hi = (M_UInt32) ~sec_hi;
       }
       else
         sec_hi = (M_UInt32)(~sec_hi+1);

       sign = 1;
     }

     if ( sec_hi < 86400 )
     {
       h2 = sec_hi;
       l1 = sec_lo / 86400;
       l2 = sec_lo % 86400;

       h3 = h2*23296;

       /* seconds remainder is in "sec_lo" */
       sec_lo = l2 + (h3 % 86400);
       if (sec_lo >= 86400)
       {
         l1++;
         sec_lo -= 86400;
       }

       /* compute number of days */
       days = (M_Int32)(h2*((M_UInt32)49710) + (M_UInt32)(h3/86400) + l1);
     }
     else
     {
       /* value is too big !! */
       days = (M_Int32)0x7FFFFFFFL;
     }

     if (sign)
     {
       days = -days;
       if ( sec_lo > 0 )
       {
         days--;
         sec_lo = 86400 - sec_lo;
       }
     }

     /* good, now compute date corresponding to this number of days */
     {
       M_DateRec  date;

       m_date_set_day_in_year( &date, days, 1970 );

       *pday   = m_date_day  ( &date );
       *pmonth = m_date_month( &date );
       *pyear  = m_date_year ( &date );
     }

     *pseconds = (M_Int32) sec_lo;
   }
  }


