#include <mlib.h>
#include MLIB_AVL_TREE_H

#define  MAX_AVL_NODES 3

  static M_Core          core;
  static M_Memory        memory;
  static M_Error         error;
  static M_Avl_NodeRec   nodes[ MAX_AVL_NODES+1 ];
  static M_Avl_Node      tree = 0;
  static M_Avl_Node      swap;  

#define  KEY(x)  M_POINTER_FROM_DIST(x)
  
  static M_Int  compare( M_Avl_Node  node,
                         M_Pointer   key )
  {
    M_Int  val1 = (M_Int)M_POINTER_TO_DIST(key);
    M_Int  val2 = node->pad;
    
    return val1 - val2;
  }                             


  static M_UInt
  node_depth( M_Avl_Node  node )
  {
    M_UInt  result = 0;
    if (node)
    {
      M_UInt  d1 = node_depth( node->links[0] );
      M_UInt  d2 = node_depth( node->links[1] );
      
      if ( d1 < d2 )
      {
        if ( d1+1 != d2 || node->balance != 1 )
          M_PANIC(( "%s: incorrect node balance", "test.avl" ));
          
        result = d2+1;
      }
      else if ( d1 > d2 )
      {
        if ( d2+1 != d1 || node->balance != -1 )
          M_PANIC(( "%s: incorrect node balance", "test.avl" ));
          
        result = d1+1;
      }
      else
      {
        if ( node->balance != 0 )
          M_PANIC(( "%s: incorrect node balance", "test.avl" ));
          
        result = d1+1;
      }
    }
    return result;
  }


  
int main( void )
{
  M_Int  n;
  
  error = m_core_new( 0, 0, 0, &core );
  if (error)
    M_PANIC(( "%s: could not create new MLib core", "test.avl" ));
    
  memory = core->memory;
    
  /* initialise nodes */
  for ( n = 0; n < MAX_AVL_NODES+1; n++ )
    nodes[n].pad = (M_UShort)n;

  /* now, insert all nodes */
  for ( n = 0; n < MAX_AVL_NODES; n++ )
  {
    swap = &nodes[n];
    m_avl_tree_insert( &tree, KEY(n), compare, &swap );
    if (swap)
      M_PANIC(( "%s: tree insertion didn't work !!\n", "test.avl" ));
  }

  /* return tree depth and check all nodes */
  M_ERROR(( "tree depth = %d, count = %d\n", node_depth( tree ), MAX_AVL_NODES ));

  /* check lookup */
  for ( n = 0; n < MAX_AVL_NODES; n++ )
  {
    if ( m_avl_tree_lookup( &tree, KEY(n), compare ) != &nodes[n] )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.avl", n ));
  }

  /* replace one node with another */
  swap = &nodes[MAX_AVL_NODES];
  m_avl_tree_insert( &tree, KEY(1), compare, &swap );
  if ( swap != &nodes[1] )
    M_PANIC(( "%s: tree replacement didn't work !!\n", "test.avl" ));

  if ( m_avl_tree_lookup( &tree, KEY(1), compare ) != &nodes[MAX_AVL_NODES] )
    M_PANIC(( "%s: tree replacement didn't work !!\n", "test.avl" ));

  for ( n = 0; n < MAX_AVL_NODES; n++ )
  {
    if ( n != 1 && m_avl_tree_lookup( &tree, KEY(n), compare ) != &nodes[n] )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.avl", n ));
  }

  /* now, remove all nodes */
  for ( n = 0; n < MAX_AVL_NODES; n++ )
  {
    swap = m_avl_tree_remove( &tree, KEY(n), compare );

    if ( n != 1 && swap != &nodes[n] )
      M_PANIC(( "%s: tree removal didn't work", "test.avl" ));

    if ( n == 1 && swap != &nodes[MAX_AVL_NODES] )
      M_PANIC(( "%s: tree removal didn't work", "test.avl" ));
  }

  if (tree)
    M_PANIC(( "tree not empty on exit" ));

  return 0;
}
