#include <mlib.h>
#include <stdio.h>

#define  M_LOG_DOMAIN     "MLIB_TESTS"
#define  M_LOG_COMPONENT  "Core"

static  M_Core     mcore;
static  M_Memory   memory;

static  M_Chain_Pool  pool;

static M_Chain
new_chain( M_CString  string )
{
  M_Chain  result;

  result = m_chain_new( pool, string );
  if (!result)
    M_PANIC(( "%s: cannot create chain from string '%s'\n",
              "mlib.test.chain",
              string ? string : "NULL" ));
  return result;
}


static M_Chain
new_chain_bound( M_CString  string,
                 M_Size     len )
{
  M_Chain  result;

  result = m_chain_new_bound( pool, string, len );
  if (!result)
    M_PANIC(( "%s: cannot create chain from bounded string '%s'\n",
              "mlib.test.chain",
              string ? string : "NULL" ));
  return result;
}


static void
test_chain_1( void )
{
  M_Chain  chain1, chain2, chain3, chain4;
  M_Byte   temp[ 40 ];

  chain1 = new_chain( 0 );
  chain2 = new_chain( "" );
  if ( chain1 != chain2 )
    M_PANIC(( "%s: invalid reference to empty chain\n",
              "mlib.test.chain" ));

  chain3 = new_chain( "mychaintest" );

  M_MEM_SET( temp, 0, sizeof(temp) );
  strncpy( (char*)temp, (char*)chain3, 40 );

  chain4 = new_chain_bound( M_CSTRING(temp), 40 );
  if ( chain4 != chain3 )
    M_PANIC(( "%s: invalid bounded chain !!\n" ));
}


int  main( void )
{
  M_Error  error;

  error = m_core_new( 0, 0, 0, &mcore );
  if (error)
    M_PANIC(( "%s: cannot create new MLib core, error = %04x\n",
              "mlib.test.chain", error ));

  if ( !mcore )
    M_PANIC(( "%s: new MLib core handle is NULL !!",
              "mlib.test.chain" ));

  memory  = mcore->memory;

  pool = m_chain_pool( mcore );
  if ( !pool )
    M_PANIC(( "%s: could not create chain pool !!",
              "mlib.test.chain" ));

  test_chain_1();

  m_core_unref( mcore );

  fprintf( stderr, "chain test: ok\n" );
  return 0;
}
