#include <mlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>

#define  M_LOG_DOMAIN     "MLIB_TESTS"
#define  M_LOG_COMPONENT  "Core"

  static const
  M_ExceptionFamilyRec   dummy_exception_family =
  {
    "dummy",
    0,
    0
  };

  static
  M_ExceptionRec   dummy_exception_rec =
  {
    &dummy_exception_family,
    1
  };

  static M_Core      core;
  static M_Error     error;

  static M_Byte*     dummy;
  static M_Exception dummy_exception = (M_Exception)&dummy_exception_rec;
  static jmp_buf     env;


  static void
  test_core_fatal_handler( M_Core       core,
                           M_Exception  exception,
                           M_Pointer    data )
  {
    M_UNUSED(core);

    if ( data != (M_Pointer)&dummy )
      M_PANIC(( "%s: received incorrect fatal data",
                "mlib.test.core.fatal_handler" ));

    if ( exception != dummy_exception )
      M_PANIC(( "%s: received incorrect fatal exception (%p instead of %p)\n",
                exception, dummy_exception ));

    M_LOG0(( "%s: triggered correctly (pseudo exception %p, value = %04x) !!\n",
             "mlib.test.core.fatal_handler", exception, exception->value ));

    longjmp( env, 1 );
  }


  typedef void (*TestFunc)( void );

  static void
  test_function( TestFunc  test_func )
  {
    if ( setjmp( env ) )
    {
      /* returned from fatal handler .. */
    }
    else
    {
      test_func();
    }
  }



  static M_Pointer   dummy_item;
  static M_Pointer   dummy_data;
  static M_UInt      dummy_count = 0;

  static void
  test_core_cleanup_dummy( M_Pointer  item,
                           M_Pointer  data )
 {
    M_LOG0(( "%s: cleaning item %p, with data %p\n",
             "mlib.test.core.cleanup.dummy", item, data ));

    if ( item != dummy_item )
      M_PANIC(( "%s: invalid item (%p instead of %p)\n",
                "mlib.test.core.cleanup.dummy", item, dummy_item ));

    if ( data != dummy_data )
      M_PANIC(( "%s: invalid data (%p instead of %p)\n",
                "mlib.test.core.cleanup.dummy", data, dummy_data ));

    dummy_count++;
  }

  static void
  test_core_push_dummy( M_Pointer  item,
                        M_Pointer  data )
  {
    dummy_item = item;
    dummy_data = data;
    m_core_push( core, item, &test_core_cleanup_dummy, data );
  }

  static void
  test_core_pop_dummy( M_Pointer    item,
                       M_PopAction  action )
  {
    m_core_pop( core, item, action );
  }




  static void
  test_core_1( void )
  {
    test_core_push_dummy( &dummy, &dummy );
    test_core_pop_dummy( &dummy, M_POP_KEEP );
  }

  static void
  test_core_2( void )
  {
    test_core_push_dummy( &dummy, &dummy );

    dummy_exception_rec.value = 0x1234;
    m_core_throw( core, dummy_exception );

    M_PANIC(( "%s: exception not thrown !!",
              "mlib.test.core.2" ));
  }


  static void
  test_core_3( void )
  {
    M_UInt  n, count = 540;

    dummy_count = 0;

    for ( n = 0; n < count; n++ )
      test_core_push_dummy( &dummy, &dummy );

    if ( dummy_count != 0 )
      M_PANIC(( "%s: invalid cleanup count after %d pushes\n",
                "mlib.test.core.3", count ));

    for ( n = 0; n < count; n++ )
      test_core_pop_dummy( &dummy, M_POP_CLEAN );

    if ( dummy_count != count )
      M_PANIC(( "%s: invalid cleanup count after %d pops\n",
                "mlib.test.core.3", count ));
  }


  static void
  test_core_4( void )
  {
    M_UInt  n, count = 540;

    dummy_count = 0;

    for ( n = 0; n < count; n++ )
      test_core_push_dummy( &dummy, &dummy );

    if ( dummy_count != 0 )
      M_PANIC(( "%s: invalid cleanup count after %d pushes\n",
                "mlib.test.core.4", count ));

    dummy_exception_rec.value = 0x2345;
    m_core_throw( core, dummy_exception );

    M_PANIC(( "%s: exception not thrown !!",
              "mlib.test.core.4" ));
  }



  int main( void )
  {
    error = m_core_new( 0, 0, 0, &core );
    if (error)
      M_PANIC(( "%s: cannot create new MLib core, error = %04x\n",
                "mlib.test.core", error ));

    if ( !core )
      M_PANIC(( "%s: new MLib core handle is NULL !!",
                "mlib.test.core" ));

    m_core_set_fatal_handler( core, &test_core_fatal_handler, &dummy );

    test_function( test_core_1 );
    test_function( test_core_2 );
    test_function( test_core_3 );
    test_function( test_core_4 );

    m_core_unref( core );


    fprintf( stderr, "test_core: ok\n" );
    exit(0);
    return 0;
  }

