#include <mlib/hash.h>

#include <stdio.h>

#define  MAX_NODES_DEFAULT 1000000

  static M_Core          core;
  static M_Memory        memory;
  static M_Error         error;
  static M_HashTable     table = 0;

  static long    max_nodes = MAX_NODES_DEFAULT;

#define  KEY(x)  M_POINTER_FROM_DIST(x)

  static M_Int  compare( M_Pointer   key1,
                         M_Pointer   key2 )
  {
    return (M_Byte*)key1 - (M_Byte*)key2;
  }


  static M_ULong  hash( M_Pointer  key )
  {
    return M_POINTER_TO_DIST(key);
  }


#if 0
  static M_UInt
  node_depth( M_Avl_Node  node )
  {
    M_UInt  result = 0;
    if (node)
    {
      M_UInt  d1 = node_depth( node->links[0] );
      M_UInt  d2 = node_depth( node->links[1] );

      if ( d1 < d2 )
      {
        if ( d1+1 != d2 || node->balance != 1 )
          M_PANIC(( "%s: incorrect node balance", "test.table" ));

        result = d2+1;
      }
      else if ( d1 > d2 )
      {
        if ( d2+1 != d1 || node->balance != -1 )
          M_PANIC(( "%s: incorrect node balance", "test.table" ));

        result = d1+1;
      }
      else
      {
        if ( node->balance != 0 )
          M_PANIC(( "%s: incorrect node balance", "test.table" ));

        result = d1+1;
      }
    }
    return result;
  }
#endif


int main( int  argc, const char** argv )
{
  M_Int  n;

  if ( argc >= 2 )
  {
    max_nodes = atoi( argv[1] );
    if ( max_nodes < 1 )
      max_nodes = 1;
  }

  error = m_core_new( 0, 0, 0, &core );
  if (error)
    M_PANIC(( "%s: could not create new MLib core", "test.table" ));

  memory = core->memory;

  /* create table */
  table = m_hash_table_new( memory, (M_HashFunc)hash, (M_CompareFunc)compare );

  /* insert all nodes */
  for ( n = 0; n < max_nodes; n++ )
    m_hash_table_set( table, KEY(n), KEY(n) );

  /* return table depth and check all nodes */
  M_ERROR(( "table size = %d\n",
             m_hash_table_get_size( table ) ));

  /* check lookup */
  for ( n = 0; n < max_nodes; n++ )
  {
    if ( m_hash_table_get( table, KEY(n) ) != KEY(n) )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.table", n ));
  }

  /* replace one node with another */
  m_hash_table_set( table, KEY(1), KEY(max_nodes) );
  if ( m_hash_table_get( table, KEY(1) ) != KEY(max_nodes) )
    M_PANIC(( "%s: table replacement didn't work !!\n", "test.table" ));

  for ( n = 0; n < max_nodes; n++ )
  {
    if ( n != 1 && m_hash_table_get( table, KEY(n) ) != KEY(n) )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.table", n ));
  }

  /* now, remove all nodes */
  for ( n = 0; n < max_nodes; n++ )
    m_hash_table_remove( table, KEY(n) );

  if ( m_hash_table_get_size(table) != 0 )
    M_PANIC(( "table not empty on exit" ));

  m_hash_table_destroy( table );

  m_core_unref( core );

  printf( "hash table test: ok !\n" );
  return 0;
}
