#include <mlib/list.h>
#include <stdio.h>

#define  M_LOG_DOMAIN     "MLIB_TESTS"
#define  M_LOG_COMPONENT  "Core"

static  M_Core       mcore;
static  M_Memory     memory;
static  M_Memory     pool;

#define  MAX_LIST  100


static void
check_list( M_List   list,
            M_Long   count )
{
  M_List  node = list;
  M_Long  n    = 0;

  for (;;)
  {
    if ( node == NULL )
    {
      if ( n == count )
        break;

      M_PANIC(( "%s: invalid list length (%d instead of %d)\n",
                "mlib.test.list.check", n, count ));
    }

    if ( n == 0 )
    {
      if ( node->prev != NULL )
        M_PANIC(( "%s: first node has a non-null 'prev' field",
                  "mlib.test.list.check" ));
    }
    else
    {
      if ( node->prev == NULL )
        M_PANIC(( "%s: non-first node has a null 'prev' field",
                  "mlib.test.list.check" ));
    }

    if ( node->next )
    {
      if ( node->next->prev != node )
        M_PANIC(( "%s: node has invalid 'next' first",
                  "mlib.test.list.check" ));
    }

    node = node->next;
    n++;
  }
}


#define  KEY(x)  M_POINTER_FROM_DIST(0x4500+(x))

static void
test_list_1( void )
{
  M_List      list1, list2, node;
  M_UInt      n;

  list1 = NULL;
  list2 = NULL;

  /* create a simple list */
  for ( n = 0; n < MAX_LIST; n++ )
    m_list_append( &list1, KEY(n), pool );

  check_list( list1, MAX_LIST );

  /* now, remove a single element */
  m_list_remove( &list1, KEY(MAX_LIST/2), pool );
  check_list( list1, MAX_LIST-1 );

  /* prepend one element */
  m_list_prepend( &list1, KEY(MAX_LIST/2), pool );
  check_list( list1, MAX_LIST );

  if ( list1->data != KEY(MAX_LIST/2) )
    M_PANIC(( "%s: prepend didn't work", "mlib.test.list.check" ));

  /* append one element */
  m_list_append( &list1, KEY(MAX_LIST), pool );
  check_list( list1, MAX_LIST+1 );

  node = m_list_last( &list1 );
  if ( node == NULL || node->data != KEY(MAX_LIST) )
    M_PANIC(( "%s: append didn't work", "mlib.test.list.check" ));

  /* remove everything */
  m_list_free_all( &list1, NULL, NULL, pool );
}




int  main( void )
{
  M_Error  error;

  error = m_core_new( 0, 0, 0, &mcore );
  if (error)
    M_PANIC(( "%s: cannot create new MLib core, error = %04x\n",
              "mlib.test.list", error ));

  if ( !mcore )
    M_PANIC(( "%s: new MLib core handle is NULL !!",
              "mlib.test.list" ));

  memory = mcore->memory;

  pool = m_list_memory_pool( memory );
  if (!pool)
    M_PANIC(( "%s: could not retrieve static list pool !!",
              "mlib.test.list" ));

  test_list_1();
  fprintf( stderr, "list test: ok\n" );

  m_core_unref( mcore );

  return 0;
}
