#include <mlib/string.h>
#include <stdio.h>

#define  M_LOG_DOMAIN     "MLIB_TESTS"
#define  M_LOG_COMPONENT  "Core"

static  M_Core     mcore;
static  M_Memory   memory;
static  M_String   s1, s2, s3;

static const M_ExceptionFamilyRec  string_exception_family =
{
  "string",
  0,
  0
};

static M_ExceptionRec  string_exception_rec =
{
  &string_exception_family,
  1
};

static M_Exception  string_exception = (M_Exception)&string_exception_rec;


static void
test_string_1( void )
{
  s1 = m_str_new( 10, memory, 0 );
  if (!s1)
    M_PANIC(( "%s: empty string !!\n", "test.string.1" ));

  s2 = m_str_dup( "this is just an example", memory, 0 );
  if (!s2)
    M_PANIC(( "%s: empty string !!\n", "test.string.1" ));

  M_XTRY
  {
    s3 = m_str_dup( s2, memory, 1 );

    if ( ! m_str_equal( s2, s3 ) )
      M_PANIC(( "%s: string copy doesn't match original ?!?\n",
                "test.string.1" ));

    m_core_throw( mcore, string_exception );
  }
  M_XCATCH(e)
  {
    M_LOG0(( "test.string.1: exception caught correctly :-)\n" ));
    if ( e != string_exception )
      M_PANIC(( "%s: invalid exception raised (%p instead of %p)\n",
                e, string_exception ));
  }
  M_XEND

  m_str_free( s2, memory );
  m_str_free( s1, memory );
}


int  main( void )
{
  M_Error  error;

  error = m_core_new( 0, 0, 0, &mcore );
  if (error)
    M_PANIC(( "%s: cannot create new MLib core, error = %04x\n",
              "mlib.test.string", error ));

  if ( !mcore )
    M_PANIC(( "%s: new MLib core handle is NULL !!",
              "mlib.test.string" ));

  memory = mcore->memory;

  test_string_1();

  m_core_unref( mcore );
  fprintf( stderr, "string test: ok\n" );
  return 0;
}
