#include <mlib/tree.h>

#include <stdio.h>

#define  MAX_NODES 100000

  static M_Core          core;
  static M_Memory        memory;
  static M_Error         error;
  static M_Tree          tree = 0;

#define  KEY(x)  M_POINTER_FROM_DIST(x)

  static M_Int  compare( M_Pointer   key1,
                         M_Pointer   key2 )
  {
    return (M_Byte*)key1 - (M_Byte*)key2;
  }



int main( void )
{
  M_Int  n;

  error = m_core_new( 0, 0, 0, &core );
  if (error)
    M_PANIC(( "%s: could not create new MLib core", "test.tree" ));

  memory = core->memory;

  /* create tree */
  tree = m_tree_new( memory, (M_CompareFunc)compare );

  /* insert all nodes */
  for ( n = 0; n < MAX_NODES; n++ )
    m_tree_set( tree, KEY(n), KEY(n) );


  /* return tree depth and check all nodes */
  M_ERROR(( "tree count = %d\n", m_tree_get_size( tree ) ));

  /* check lookup */
  for ( n = 0; n < MAX_NODES; n++ )
  {
    if ( m_tree_get( tree, KEY(n) ) != KEY(n) )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.tree", n ));
  }

  /* replace one node with another */
  m_tree_set( tree, KEY(1), KEY(MAX_NODES) );
  if ( m_tree_get( tree, KEY(1) ) != KEY(MAX_NODES) )
    M_PANIC(( "%s: tree replacement didn't work !!\n", "test.tree" ));

  for ( n = 0; n < MAX_NODES; n++ )
  {
    if ( n != 1 && m_tree_get( tree, KEY(n) ) != KEY(n) )
      M_PANIC(( "%s: couldn't lookup element # %d", "test.tree", n ));
  }

  /* now, remove all nodes */
  for ( n = 0; n < MAX_NODES; n++ )
    m_tree_remove( tree, KEY(n) );

  if ( m_tree_get_size(tree) != 0 )
    M_PANIC(( "tree not empty on exit" ));

  m_tree_destroy( tree );

  m_core_unref( core );

  printf( "tree test: ok !\n" );
  return 0;
}
